import pytz
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _


def timezone_validator(value):
    """Timezone validator."""
    try:
        return pytz.timezone(value)
    except (pytz.UnknownTimeZoneError, AttributeError):
        raise ValidationError(
            _('Timezone validation error: %(value)s'),
            code='timezone_error',
            params={'value': value}
        )



"""Downloader class tests."""
import tempfile
import time
from unittest import mock
import logging

from django import test

from cities_light.downloader import Downloader
from cities_light.exceptions import SourceFileDoesNotExist

import builtins as b  # do not remove


class TestDownloader(test.TransactionTestCase):
    """Downloader tests."""

    logger = logging.getLogger('cities_light')

    @mock.patch('cities_light.downloader.os.path.exists')
    def test_source_matches_destination(self, mock_func):
        """Tests for source_matches_destination behavior."""
        mock_func.return_value = True
        downloader = Downloader()
        # Different destination
        source = 'file:///a.txt'
        dest = '/b.txt'
        self.assertFalse(
            downloader.source_matches_destination(source, dest)
        )
        # Same destination with same file name
        source = 'file:///data/a.txt'
        dest = '/data/a.txt'
        self.assertTrue(
            downloader.source_matches_destination(source, dest)
        )
        # Different destination with same file name
        source = 'http://server/download/data/a.txt'
        dest = '/data/a.txt'
        self.assertFalse(
            downloader.source_matches_destination(source, dest)
        )

        mock_func.return_value = False
        # Exception handling, checking whether file exist or not,
        # if exist then checking source and destination
        source = 'file:///data/a.txt'
        dest = '/data/a.txt'
        with self.assertRaises(SourceFileDoesNotExist):
            downloader.source_matches_destination(source, dest)

    @mock.patch('cities_light.downloader.time.gmtime')
    @mock.patch('cities_light.downloader.os.path.getmtime')
    @mock.patch('cities_light.downloader.os.path.getsize')
    @mock.patch('cities_light.downloader.os.path.exists')
    def test_needs_downloading(self, *args):
        """Tests for needs_downloading behavior."""
        m_urlopen = mock.Mock(headers={
            'last-modified': 'Sat, 02 Jan 2016 00:04:14 GMT',
            'content-length': '13469'
        })

        loc_exists = args[0]
        loc_getsize = args[1]
        loc_getmtime = args[2]
        loc_gmtime = args[3]

        loc_getmtime.return_value = 1
        loc_exists.return_value = True
        destination = '/data/abc'
        downloader = Downloader()
        with mock.patch('cities_light.downloader.urlopen',
                        return_value=m_urlopen):
            # Source and local time and size's equal
            loc_gmtime.return_value = time.strptime(
                '02-01-2016 00:04:14 GMT', '%d-%m-%Y %H:%M:%S %Z')
            loc_getsize.return_value = 13469
            params = {
                'source': 'file:///a.txt',
                'destination': destination,
                'force': False}
            result = downloader.needs_downloading(**params)
            self.assertFalse(result)

            # Destination time > source time, size is equal
            loc_gmtime.return_value = time.strptime(
                '02-01-2016 00:04:15 GMT', '%d-%m-%Y %H:%M:%S %Z')
            loc_getsize.return_value = 13469
            params = {
                'source': 'file:///a.txt',
                'destination': destination,
                'force': False}
            result = downloader.needs_downloading(**params)
            self.assertFalse(result)

            # Destination time < source time, size is equal
            loc_gmtime.return_value = time.strptime(
                '02-01-2016 00:04:13 GMT', '%d-%m-%Y %H:%M:%S %Z')
            loc_getsize.return_value = 13469
            params = {
                'source': 'file:///a.txt',
                'destination': destination,
                'force': False}
            result = downloader.needs_downloading(**params)
            self.assertTrue(result)

            # Source and destination time is equal,
            # source and destination size is not equal
            loc_gmtime.return_value = time.strptime(
                '02-01-2016 00:04:14 GMT', '%d-%m-%Y %H:%M:%S %Z')
            loc_getsize.return_value = 13470
            params = {
                'source': 'file:///a.txt',
                'destination': destination,
                'force': False}
            result = downloader.needs_downloading(**params)
            self.assertTrue(result)

            # Source and destination have the same time and size
            # force = True
            loc_gmtime.return_value = time.strptime(
                '02-01-2016 00:04:14 GMT', '%d-%m-%Y %H:%M:%S %Z')
            loc_getsize.return_value = 13469
            params = {
                'source': 'file:///a.txt',
                'destination': destination,
                'force': True}
            result = downloader.needs_downloading(**params)
            self.assertTrue(result)

            # Destination file does not exist
            loc_exists.return_value = False
            loc_gmtime.return_value = time.strptime(
                '02-01-2016 00:04:14 GMT', '%d-%m-%Y %H:%M:%S %Z')
            loc_getsize.return_value = 13470
            params = {
                'source': 'file:///a.txt',
                'destination': destination,
                'force': False}
            result = downloader.needs_downloading(**params)
            self.assertTrue(result)

    @mock.patch.object(Downloader, 'source_matches_destination')
    def test_download_calls_source_matches_destination(self, m_check):
        """Test if download() checks for source and destination match."""
        m_check.return_value = True
        downloader = Downloader()
        source = 'file:///a.txt'
        destination = '/a.txt'
        # The downloader.download will return false
        # as source and destination are same
        # The downloader.source_matches_destination will return
        # true and downloader.download will return false
        self.assertFalse(
            downloader.download(
                source,
                destination,
                False
            )
        )
        m_check.assert_called_with(source, destination)

    @mock.patch.object(Downloader, 'needs_downloading')
    @mock.patch.object(Downloader, 'source_matches_destination')
    def test_download_calls_needs_downloading(self, m_check, m_need):
        """Test if download() checks if source should be downloaded."""
        m_check.return_value = False
        m_need.return_value = False
        downloader = Downloader()
        source = 'file:///a.txt'
        destination = '/a.txt'
        # Here dowaloder.needs_downloading() will return false
        # as the time of modifiaction of dest>= time of source
        # and the size od source and destination are same
        # and downloader.download will return false
        self.assertFalse(
            downloader.download(
                source,
                destination,
                False
            )
        )
        m_check.assert_called_with(source, destination)
        m_need.assert_called_with(source, destination, False)

    @mock.patch.object(Downloader, 'needs_downloading')
    @mock.patch.object(Downloader, 'source_matches_destination')
    def test_download(self, m_check, m_need):
        """Test actual download."""
        m_check.return_value = False
        m_need.return_value = True
        downloader = Downloader()
        source = 'file:///b.txt'
        destination = '/a.txt'

        tmpfile = tempfile.SpooledTemporaryFile(max_size=1024000, mode='wb')
        tmpfile.write(b'source content')
        tmpfile.seek(0)

        with mock.patch('cities_light.downloader.urlopen',
                        return_value=tmpfile):
            module_name = '{}.b.open'.format(__name__)
            mock_open = mock.mock_open()
            # The downoader.needs_downloading will return true and last three
            # lines of downloader.download will copy the source to sestination
            with mock.patch(module_name, mock_open):
                self.assertTrue(downloader.download(
                    source,
                    destination,
                    False))
                handle = mock_open()
                handle.write.assert_called_once_with(b'source content')

    def test_not_download(self):
        """Tests actual not download."""
        with mock.patch.object(Downloader, 'source_matches_destination') as m:
            m.return_value = True
            downloader = Downloader()
            source = 'file:///b.txt'
            destination = '/a.txt'
            with mock.patch('cities_light.downloader.urlopen') as uo_mock:
                downloader.download(source, destination)
                uo_mock.assert_not_called()

        with mock.patch.object(Downloader, 'source_matches_destination') as m:
            m.return_value = False
            with mock.patch.object(Downloader, 'needs_downloading') as n:
                n.return_value = False
                downloader = Downloader()
                source = 'file:///b.txt'
                destination = '/a.txt'
                # Here copy of b has been made in above function,the
                # downloder.needs_downloading() will return false
                # and no download will happen
                with mock.patch('cities_light.downloader.urlopen') as uo_mock:
                    downloader.download(source, destination)
                    uo_mock.assert_not_called()



class CitiesLightException(Exception):
    """ Base exception class for this app's exceptions. """
    pass


class InvalidItems(CitiesLightException):
    """
    The cities_light command will skip item if a city_items_pre_import signal
    reciever raises this exception.
    """
    pass


class SourceFileDoesNotExist(CitiesLightException):
    """ A source file could not be found. """

    def __init__(self, source):
        super().__init__('%s does not exist' % source)



from django.db.models import signals
from .abstract_models import to_ascii, to_search
from .settings import *
from .signals import *
from .exceptions import *


def set_name_ascii(sender, instance=None, **kwargs):
    """
    Signal reciever that sets instance.name_ascii from instance.name.

    Ascii versions of names are often useful for autocompletes and search.
    """
    name_ascii = to_ascii(instance.name).strip()

    if name_ascii and not instance.name_ascii:
        instance.name_ascii = name_ascii


def set_display_name(sender, instance=None, **kwargs):
    """
    Set instance.display_name to instance.get_display_name(), avoid spawning
    queries during __str__().
    """
    instance.display_name = instance.get_display_name()


def city_country(sender, instance, **kwargs):
    if instance.region_id and not instance.country_id:
        instance.country = instance.region.country


def city_search_names(sender, instance, **kwargs):
    search_names = set()

    country_names = {instance.country.name, }
    if instance.country.alternate_names:
        for n in instance.country.alternate_names.split(';'):
            country_names.add(n)

    city_names = {instance.name, }
    if instance.alternate_names:
        for n in instance.alternate_names.split(';'):
            city_names.add(n)

    if instance.region_id:
        region_names = {instance.region.name, }
        if instance.region.alternate_names:
            for n in instance.region.alternate_names.split(';'):
                region_names.add(n)
    else:
        region_names = set()

    for city_name in city_names:
        for country_name in country_names:
            name = to_search(city_name + country_name)
            search_names.add(name)

            for region_name in region_names:
                name = to_search(city_name + region_name + country_name)
                search_names.add(name)

    instance.search_names = ' '.join(sorted(search_names))


def connect_default_signals(model_class):
    """
    Use this function to connect default signals to your custom model.
    It is called automatically, if default cities_light models are used,
    i.e. settings `CITIES_LIGHT_APP_NAME` is not changed.
    """
    if 'Country' in model_class.__name__:
        signals.pre_save.connect(set_name_ascii, sender=model_class)
    elif 'SubRegion' in model_class.__name__:
        signals.pre_save.connect(set_name_ascii, sender=model_class)
        signals.pre_save.connect(set_display_name, sender=model_class)
    elif 'Region' in model_class.__name__:
        signals.pre_save.connect(set_name_ascii, sender=model_class)
        signals.pre_save.connect(set_display_name, sender=model_class)
    elif 'City' in model_class.__name__:
        signals.pre_save.connect(set_name_ascii, sender=model_class)
        signals.pre_save.connect(set_display_name, sender=model_class)
        signals.pre_save.connect(city_country, sender=model_class)
        signals.pre_save.connect(city_search_names, sender=model_class)


def filter_non_cities(sender, items, **kwargs):
    """
    Exclude any **city** which feature code must not be included.
    By default, this receiver is connected to
    :py:func:`~cities_light.signals.city_items_pre_import`, it raises
    :py:class:`~cities_light.exceptions.InvalidItems` if the row feature code
    is not in the :py:data:`~cities_light.settings.INCLUDE_CITY_TYPES` setting.
    """
    if items[7] not in INCLUDE_CITY_TYPES:
        raise InvalidItems()


city_items_pre_import.connect(filter_non_cities)


def filter_non_included_countries_country(sender, items, **kwargs):
    """
    Exclude any **country** which country must not be included.
    This is slot is connected to the
    :py:func:`~cities_light.signals.country_items_pre_import` signal and does
    nothing by default.  To enable it, set the
    :py:data:`~cities_light.settings.INCLUDE_COUNTRIES` setting.
    """
    if INCLUDE_COUNTRIES is None:
        return

    if items[0].split('.')[0] not in INCLUDE_COUNTRIES:
        raise InvalidItems()


country_items_pre_import.connect(filter_non_included_countries_country)


def filter_non_included_countries_region(sender, items, **kwargs):
    """
    Exclude any **region** which country must not be included.
    This is slot is connected to the
    :py:func:`~cities_light.signals.region_items_pre_import` signal and does
    nothing by default.  To enable it, set the
    :py:data:`~cities_light.settings.INCLUDE_COUNTRIES` setting.
    """
    if INCLUDE_COUNTRIES is None:
        return

    if items[0].split('.')[0] not in INCLUDE_COUNTRIES:
        raise InvalidItems()


region_items_pre_import.connect(filter_non_included_countries_region)


def filter_non_included_countries_subregion(sender, items, **kwargs):
    """
    Exclude any **subregion** which country must not be included.
    This is slot is connected to the
    :py:func:`~cities_light.signals.subregion_items_pre_import` signal and does
    nothing by default.  To enable it, set the
    :py:data:`~cities_light.settings.INCLUDE_COUNTRIES` setting.
    """
    if INCLUDE_COUNTRIES is None:
        return

    if items[0].split('.')[0] not in INCLUDE_COUNTRIES:
        raise InvalidItems()


subregion_items_pre_import.connect(filter_non_included_countries_subregion)


def filter_non_included_countries_city(sender, items, **kwargs):
    """
    Exclude any **city** which country must not be included.
    This is slot is connected to the
    :py:func:`~cities_light.signals.city_items_pre_import` signal and does
    nothing by default.  To enable it, set the
    :py:data:`~cities_light.settings.INCLUDE_COUNTRIES` setting.
    """
    if INCLUDE_COUNTRIES is None:
        return

    if items[8].split('.')[0] not in INCLUDE_COUNTRIES:
        raise InvalidItems()


city_items_pre_import.connect(filter_non_included_countries_city)



"""Test for cities_light_fixtures management command."""
import bz2
import os
from unittest import mock

from django import test
from django.core.management import call_command
from django.core.management.base import CommandError

from dbdiff.fixture import Fixture
from cities_light.settings import DATA_DIR, FIXTURES_BASE_URL
from cities_light.management.commands.cities_light_fixtures import Command
from cities_light.downloader import Downloader
from cities_light.models import City
from .base import FixtureDir


class TestCitiesLigthFixtures(test.TransactionTestCase):
    """Tests for cities_light_fixtures management command."""

    def test_dump_fixtures(self):
        """
        Test dump_fixtures calls dump_fixture with Country,
        Region, Subregion and City table names.
        """
        fixtures_dir = os.path.join(DATA_DIR, 'fixtures')
        with mock.patch.object(Command, 'dump_fixture') as mock_func:
            cmd = Command()
            cmd.country_path = os.path.join(fixtures_dir, cmd.COUNTRY_FIXTURE)
            cmd.region_path = os.path.join(fixtures_dir, cmd.REGION_FIXTURE)
            cmd.subregion_path = os.path.join(fixtures_dir, cmd.SUBREGION_FIXTURE)
            cmd.city_path = os.path.join(fixtures_dir, cmd.CITY_FIXTURE)
            cmd.dump_fixtures()
            mock_func.assert_any_call('cities_light.Country', cmd.country_path)
            mock_func.assert_any_call('cities_light.Region', cmd.region_path)
            mock_func.assert_any_call('cities_light.SubRegion', cmd.subregion_path)
            mock_func.assert_any_call('cities_light.City', cmd.city_path)

    def test_dump_fixture(self):
        """
        Test dump_fixture calls dumpdata management command
        and tries to save it to file."""
        # Load test data
        destination = FixtureDir('import').get_file_path('angouleme.json')
        call_command('loaddata', destination)
        # Dump
        try:
            fixture_path = os.path.join(os.path.dirname(__file__), "fixtures", "test_dump_fixture.json")
            cmd = Command()
            cmd.dump_fixture('cities_light.City', fixture_path, True)
            with bz2.BZ2File(fixture_path, mode='r') as bzfile:
                data = bzfile.read()
            with open(fixture_path, mode='wb') as file:
                file.write(data)
            Fixture(fixture_path, models=[City]).assertNoDiff()
        finally:
            if os.path.exists(fixture_path):
                os.remove(fixture_path)

    def test_load_fixtures(self):
        """
        Test load_fixtures calls load_fixture with country,
        region, subregion and city path.
        """
        fixtures_dir = os.path.join(DATA_DIR, 'fixtures')
        with mock.patch.object(Command, 'load_fixture') as mock_func:
            cmd = Command()
            # paths
            cmd.country_path = os.path.join(fixtures_dir, cmd.COUNTRY_FIXTURE)
            cmd.region_path = os.path.join(fixtures_dir, cmd.REGION_FIXTURE)
            cmd.subregion_path = os.path.join(fixtures_dir, cmd.SUBREGION_FIXTURE)
            cmd.city_path = os.path.join(fixtures_dir, cmd.CITY_FIXTURE)
            # URLs
            cmd.country_url = FIXTURES_BASE_URL + cmd.COUNTRY_FIXTURE
            cmd.region_url = FIXTURES_BASE_URL + cmd.REGION_FIXTURE
            cmd.subregion_url = FIXTURES_BASE_URL + cmd.SUBREGION_FIXTURE
            cmd.city_url = FIXTURES_BASE_URL + cmd.CITY_FIXTURE

            cmd.load_fixtures(force_fetch=True)
            mock_func.assert_any_call(
                cmd.country_url, cmd.country_path, force=True)
            mock_func.assert_any_call(
                cmd.region_url, cmd.region_path, force=True)
            mock_func.assert_any_call(
                cmd.subregion_url, cmd.subregion_path, force=True)
            mock_func.assert_any_call(
                cmd.city_url, cmd.city_path, force=True)

    def test_load_fixture(self):
        """Test loaded fixture matches database content."""
        destination = FixtureDir('import').get_file_path('angouleme.json')
        with mock.patch.object(Downloader, 'download') as mock_func:
            cmd = Command()
            cmd.load_fixture(source='/abcdefg.json',
                             destination=destination,
                             force=True)
            Fixture(destination).assertNoDiff()
            mock_func.assert_called_with(source='/abcdefg.json',
                                         destination=destination,
                                         force=True)

    @mock.patch('cities_light.downloader.os.path.exists')
    def test_incorrect_subcommand(self, m_exists):
        """Test cities_light_fixtures fails on unsupported command."""
        m_exists.return_value = True
        with self.assertRaises(CommandError) as error:
            call_command('cities_light_fixtures', 'brrr')
        self.assertEqual(str(error.exception),
                         'Please specify either "load" or "dump" command')

    @mock.patch('cities_light.downloader.os.path.exists')
    def test_call_with_load_argument(self, m_exists):
        """Test 'cities_light_fixtures load' calls load method."""
        m_exists.return_value = True
        module = ('cities_light.management.commands.'
                  'cities_light_fixtures.Command.load_fixtures')
        with mock.patch(module) as mock_func:
            call_command('cities_light_fixtures', 'load')
            self.assertTrue(mock_func.called)

    @mock.patch('cities_light.downloader.os.path.exists')
    def test_call_with_dump_argument(self, m_exists):
        """Test 'cities_light_fixtures dump' calls dump method."""
        m_exists.return_value = True
        module = ('cities_light.management.commands.'
                  'cities_light_fixtures.Command.dump_fixtures')
        with mock.patch(module) as mock_func:
            call_command('cities_light_fixtures', 'dump')
            self.assertTrue(mock_func.called)

    @mock.patch('cities_light.downloader.os.path.exists')
    def test_call_with_base_url_and_fixtures_base_url_is_none(self, m_exists):
        """Test cities_light_fixtures --base-url option handling."""
        m_exists.return_value = True

        module = 'cities_light.management.commands.cities_light_fixtures'
        func = '{0}.{1}'.format(module, 'Command.load_fixtures')
        setting = '{0}.{1}'.format(module, 'FIXTURES_BASE_URL')

        with mock.patch(setting, None):
            # --base-url not specified
            with mock.patch(func) as _mock:
                exc = 'Please specify --base-url or settings'
                with self.assertRaisesRegex(CommandError, exc):
                    call_command('cities_light_fixtures', 'load')
            _mock.assert_not_called()

            # --base-url
            with mock.patch(func) as _mock:
                params = {'base_url': 'path/to/fixtures/for/load'}
                call_command('cities_light_fixtures', 'load', **params)
            self.assertTrue(_mock.called)



from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0004_autoslug_update'),
    ]

    operations = [
        migrations.AlterField(
            model_name='country',
            name='phone',
            field=models.CharField(max_length=20, null=True, blank=True),
        ),
    ]



from django.contrib.auth import get_user_model
from django.test import TestCase
from django.test.client import RequestFactory
from django.db.models import query
from django.contrib.admin.sites import AdminSite

from cities_light import admin as cl_admin
from cities_light import models as cl_models


class AdminTestCase(TestCase):
    def setUp(self):
        self.factory = RequestFactory()
        self.admin_site = AdminSite()

    def testCityChangeList(self):
        user = get_user_model().objects.create(is_superuser=True, username="test")
        request = self.factory.get('/some/path/', data={'q': 'some query'})
        request.user = user
        city_admin = cl_admin.CityAdmin(cl_models.City, self.admin_site)
        changelist = cl_admin.CityChangeList(
            request, cl_models.City, cl_admin.CityAdmin.list_display,
            cl_admin.CityAdmin.list_display_links, cl_admin.CityAdmin.list_filter,
            cl_admin.CityAdmin.date_hierarchy, cl_admin.CityAdmin.search_fields,
            cl_admin.CityAdmin.list_select_related, cl_admin.CityAdmin.list_per_page,
            cl_admin.CityAdmin.list_max_show_all, cl_admin.CityAdmin.list_editable, city_admin, "id")

        self.assertIsInstance(changelist.get_queryset(request), query.QuerySet)



import os.path
import zipfile
import logging

from .settings import *
from .downloader import Downloader


class Geonames:
    logger = logging.getLogger('cities_light')

    def __init__(self, url: str, force: bool = False):
        # Creating a directory if not exist
        if not os.path.exists(DATA_DIR):
            self.logger.info('Creating %s', DATA_DIR)
            os.mkdir(DATA_DIR)

        destination_file_name = url.split('/')[-1]
        self.file_path = os.path.join(DATA_DIR, destination_file_name)

        self.downloaded = self.download(
            url=url,
            path=self.file_path,
            force=force
        )

        # Extract the destination file, use the extracted file as new
        # destination
        destination_file_name = destination_file_name.replace(
            'zip', 'txt')

        destination = os.path.join(DATA_DIR, destination_file_name)
        exists = os.path.exists(destination)
        # If the file is a zipped file then extract it
        if url.split('.')[-1] == 'zip' and not exists:
            self.extract(self.file_path, destination_file_name)
        self.file_path = os.path.join(
            DATA_DIR, destination_file_name)

    @staticmethod
    def download(url, path, force=False):
        downloader = Downloader()
        # Returns true or false(either downloaded or not based on
        # the condition in downloader.py
        return downloader.download(
            source=url,
            destination=path,
            force=force
        )

    def extract(self, zip_path, file_name):
        destination = os.path.join(DATA_DIR, file_name)

        self.logger.info('Extracting %s from %s into %s',
                         file_name, zip_path, destination)
        # Extracting the file in the data directory
        zip_file = zipfile.ZipFile(zip_path)
        if zip_file:
            zip_file.extract(file_name, DATA_DIR)

    def parse(self):
        file = open(self.file_path, encoding='utf-8', mode='r')

        for line in file:
            line = line.strip()
            # If the line is blank/empty or a comment, skip it and continue
            if len(line) < 1 or line[0] == '#':
                continue
            # Split on tab character and strip the new line character
            yield [e.strip() for e in line.split('\t')]

    def num_lines(self):
        return sum(1 for line in open(self.file_path, encoding='utf-8'))



from django import test

from ..forms import CountryForm, CityForm
from ..models import Country, City


class FormTestCase(test.TransactionTestCase):
    reset_sequences = True

    def testCountryFormNameAndContinentAlone(self):
        form = CountryForm({'name': 'Spain', 'continent': 'EU'})
        self.assertTrue(form.is_valid())
        form.save()

    def testCityFormNameAndCountryAlone(self):
        country = Country(name='France')
        country.save()
        form = CityForm({'name': 'Paris', 'country': country.pk})
        self.assertTrue(form.is_valid())
        form.save()


class SaveTestCase(test.TransactionTestCase):
    reset_sequences = True

    def testCountryAsciiAndSlug(self):
        country = Country(name='áó éú', geoname_id=1)
        country.save()

        self.assertEqual(country.name_ascii, 'ao eu')
        self.assertEqual(country.slug, 'ao-eu')

    def testCityAsciiAndSlug(self):
        country = Country(name='Belgium', geoname_id=2802361)
        country.save()

        city = City(name='áó éú', country=country, geoname_id=2)
        city.save()

        self.assertEqual(city.name_ascii, 'ao eu')
        self.assertEqual(city.slug, 'ao-eu')



"""
Django settings for test_project.

Generated by 'django-admin startproject' using Django 1.10.2.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

import os
import sys

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = 'mzdvd*#0=$g(-!v_vj_7^(=zrh3klia(u&amp;cqd3nr7p^khh^ui#'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = []

# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'cities_light',
]

# Rename to MIDDLEWARE on Django 1.10
MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
]

ROOT_URLCONF = 'urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'wsgi.application'

# Database
# https://docs.djangoproject.com/en/1.10/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.' + os.environ.get('DB_ENGINE', 'sqlite3'),
        'HOST': os.environ.get('DB_HOST', ''),
        'NAME': os.environ.get('DB_NAME', 'db.sqlite'),
        'USER': os.environ.get('DB_USER', ''),
        'PASSWORD': os.environ.get('DB_PASSWORD', ''),
        'PORT': os.environ.get('DB_PORT', ''),
        'OPTIONS': {},
    }
}

if sys.version_info[0] < 3 and 'mysql' in DATABASES['default']['ENGINE']:
    DATABASES['default']['OPTIONS']['autocommit'] = True
if 'mysql' in DATABASES['default']['ENGINE']:
    DATABASES['default']['OPTIONS']['charset'] = 'utf8mb4'
    DATABASES['default']['TEST'] = {
        'CHARSET': 'utf8mb4',
        'COLLATION': 'utf8mb4_unicode_ci',
    }

# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/

STATIC_URL = '/static/'

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error when DEBUG=False.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'filters': {
        'require_debug_false': {
            '()': 'django.utils.log.RequireDebugFalse'
        }
    },
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'filters': ['require_debug_false'],
            'class': 'django.utils.log.AdminEmailHandler'
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'DEBUG',
        },
        'cities_light': {
            'handlers': ['console'],
            'propagate': True,
            'level': 'INFO',
        },
    }
}

if os.environ.get('CI', False):
    CITIES_LIGHT_TRANSLATION_LANGUAGES = ['fr', 'ru']

    FIXTURE_DIR = os.path.abspath(
        os.path.join(BASE_DIR, 'src', 'cities_light', 'tests', 'fixtures')
    )

    FIXTURE_DIRS = [FIXTURE_DIR]

    LOGGING['loggers']['cities_light']['level'] = 'DEBUG'

    INSTALLED_APPS += ('dbdiff',)



from django.db import models, migrations
import autoslug.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name_ascii', models.CharField(db_index=True, max_length=200, blank=True)),
                ('slug', autoslug.fields.AutoSlugField(editable=False)),
                ('geoname_id', models.IntegerField(unique=True, null=True, blank=True)),
                ('alternate_names', models.TextField(default='', null=True, blank=True)),
                ('name', models.CharField(unique=True, max_length=200)),
                ('code2', models.CharField(max_length=2, unique=True, null=True, blank=True)),
                ('code3', models.CharField(max_length=3, unique=True, null=True, blank=True)),
                ('continent', models.CharField(db_index=True, max_length=2, choices=[('OC', 'Oceania'), ('EU', 'Europe'), ('AF', 'Africa'), ('NA', 'North America'), ('AN', 'Antarctica'), ('SA', 'South America'), ('AS', 'Asia')])),
                ('tld', models.CharField(db_index=True, max_length=5, blank=True)),
                ('phone', models.CharField(max_length=20, null=True)),
            ],
            options={
                'ordering': ['name'],
                'abstract': False,
                'verbose_name_plural': 'countries',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Region',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name_ascii', models.CharField(db_index=True, max_length=200, blank=True)),
                ('slug', autoslug.fields.AutoSlugField(editable=False)),
                ('geoname_id', models.IntegerField(unique=True, null=True, blank=True)),
                ('alternate_names', models.TextField(default='', null=True, blank=True)),
                ('name', models.CharField(max_length=200, db_index=True)),
                ('display_name', models.CharField(max_length=200)),
                ('geoname_code', models.CharField(db_index=True, max_length=50, null=True, blank=True)),
                ('country', models.ForeignKey(to='cities_light.Country', to_field='id', on_delete=models.CASCADE)),
            ],
            options={
                'ordering': ['name'],
                'unique_together': set([('country', 'name'), ('country', 'slug')]),
                'abstract': False,
                'verbose_name': 'region/state',
                'verbose_name_plural': 'regions/states',
            },
            bases=(models.Model,),
        ),
    ]



# Generated by Django 3.0.5 on 2020-05-08 18:51

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0009_add_subregion'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='city',
            unique_together={('region', 'subregion', 'slug'), ('region', 'subregion', 'name')},
        ),
        migrations.AlterField(
            model_name='subregion',
            name='region',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                    to='cities_light.Region'),
        ),
    ]



"""
Signals for this application.

.. py:data:: city_items_pre_import

    Emited by city_import() in the cities_light command for each row parsed in
    the data file. If a signal reciever raises InvalidItems then it will be
    skipped.

    An example is worth 1000 words: if you want to import only cities from
    France, USA and Belgium you could do as such::

        import cities_light

        def filter_city_import(sender, items, **kwargs):
            if items[8] not in ('FR', 'US', 'BE'):
                raise cities_light.InvalidItems()

        cities_light.signals.city_items_pre_import.connect(filter_city_import)

    Note: this signal gets a list rather than a City instance for performance
    reasons.

.. py:data:: region_items_pre_import

    Same as :py:data:`~cities_light.signals.city_items_pre_import`.

.. py:data:: country_items_pre_import

    Same as :py:data:`~cities_light.signals.region_items_pre_import` and
    :py:data:`cities_light.signals.city_items_pre_import`.

.. py:data:: translation_items_pre_import

    Same as :py:data:`~cities_light.signals.region_items_pre_import` and
    :py:data:`cities_light.signals.city_items_pre_import`.

    Note: Be careful because of long runtime; it will be called VERY often.

.. py:data:: city_items_post_import

    Emited by city_import() in the cities_light command for each row parsed in
    the data file, right before saving City object. Along with City instance
    it pass items with geonames data. Will be useful, if you define custom
    cities models with ``settings.CITIES_LIGHT_APP_NAME``.

    Example::

        import cities_light

        def process_city_import(sender, instance, items, **kwargs):
            instance.timezone = items[17]

        cities_light.signals.city_items_post_import.connect(process_city_import)

.. py:data:: region_items_post_import

    Same as :py:data:`~cities_light.signals.city_items_post_import`.

.. py:data:: country_items_post_import

    Same as :py:data:`~cities_light.signals.region_items_post_import` and
    :py:data:`cities_light.signals.city_items_post_import`.
"""
import django.dispatch

__all__ = [
    'country_items_pre_import', 'country_items_post_import',
    'region_items_pre_import', 'region_items_post_import',
    'subregion_items_pre_import', 'subregion_items_post_import',
    'city_items_pre_import', 'city_items_post_import',
    'translation_items_pre_import']

# providing_args=['items'] for signals below
city_items_pre_import = django.dispatch.Signal()
subregion_items_pre_import = django.dispatch.Signal()
region_items_pre_import = django.dispatch.Signal()
country_items_pre_import = django.dispatch.Signal()
translation_items_pre_import = django.dispatch.Signal()

# providing_args=['instance', 'items'] for all signals below
city_items_post_import = django.dispatch.Signal()
subregion_items_post_import = django.dispatch.Signal()
region_items_post_import = django.dispatch.Signal()
country_items_post_import = django.dispatch.Signal()



from django.db import migrations, models
import cities_light.abstract_models

from cities_light.settings import INDEX_SEARCH_NAMES


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0005_blank_phone'),
    ]

    operations = [
        migrations.AlterField(
            model_name='city',
            name='search_names',
            field=cities_light.abstract_models.ToSearchTextField(default='', blank=True, max_length=4000, db_index=INDEX_SEARCH_NAMES),
        ),
    ]



"""Data downloader."""

import logging
import time
import os

from urllib.request import urlopen
from urllib.parse import urlparse

from .exceptions import SourceFileDoesNotExist


class Downloader:
    """Geonames data downloader class."""

    def download(self, source: str, destination: str, force: bool = False):
        """Download source file/url to destination."""
        logger = logging.getLogger('cities_light')

        # Prevent copying itself
        # If same file then return
        if self.source_matches_destination(source, destination):
            logger.warning('Download source matches destination file')
            return False
        # Checking if download is needed i.e. names are different but
        # they are same file essentiallly
        # If needed continue else return.
        if not self.needs_downloading(source, destination, force):
            logger.warning(
                'Assuming local download is up to date for %s', source)
            return False
        # If the files are different, download/copy happens
        logger.info('Downloading %s into %s', source, destination)
        source_stream = urlopen(source)
        # wb: open as write and binary mode
        with open(destination, 'wb') as local_file:
            local_file.write(source_stream.read())

        return True

    @staticmethod
    def source_matches_destination(source: str, destination: str):
        """Return True if source and destination point to the same file."""
        parsed_source = urlparse(source)
        if parsed_source.scheme == 'file':
            source_path = os.path.abspath(os.path.join(parsed_source.netloc,
                                                       parsed_source.path))
            # Checking exception of file exist or not
            if not os.path.exists(source_path):
                raise SourceFileDoesNotExist(source_path)

            if source_path == destination:
                return True
        return False

    @staticmethod
    def needs_downloading(source: str, destination: str, force: bool):
        """Return True if source should be downloaded to destination."""
        src_file = urlopen(source)
        src_size = int(src_file.headers['content-length'])
        # Getting last modified timestamp
        src_last_modified = time.strptime(
            src_file.headers['last-modified'],
            '%a, %d %b %Y %H:%M:%S %Z'  # taking time with second
        )

        if os.path.exists(destination) and not force:
            local_time = time.gmtime(os.path.getmtime(destination))
            local_size = os.path.getsize(destination)
            # Checking the timestamp of creation and the file size,
            # if destination timestamp is equal or greater and the size
            # is also equal then return falase as no need to download
            if local_time >= src_last_modified and local_size == src_size:
                return False
        return True



"""
Couple djangorestframework and cities_light.

It defines a urlpatterns variables, with the following urls:

- cities-light-api-city-list
- cities-light-api-city-detail
- cities-light-api-region-list
- cities-light-api-region-detail
- cities-light-api-country-list
- cities-light-api-country-detail

If rest_framework (v3) is installed, all you have to do is add this url
include::

    path('cities_light/api/',
         include('cities_light.contrib.restframework3')),

And that's all !
"""
from django.urls import include, path
from rest_framework import viewsets, relations
from rest_framework.serializers import HyperlinkedModelSerializer
from rest_framework import routers


from ..loading import get_cities_models

Country, Region, SubRegion, City = get_cities_models()


class CitySerializer(HyperlinkedModelSerializer):
    """
    HyperlinkedModelSerializer for City.
    """
    url = relations.HyperlinkedIdentityField(
        view_name='cities-light-api-city-detail')
    country = relations.HyperlinkedRelatedField(
        view_name='cities-light-api-country-detail', read_only=True)
    region = relations.HyperlinkedRelatedField(
        view_name='cities-light-api-region-detail', read_only=True)
    subregion = relations.HyperlinkedRelatedField(
        view_name='cities-light-api-subregion-detail', read_only=True)

    class Meta:
        model = City
        exclude = ('slug',)


class SubRegionSerializer(HyperlinkedModelSerializer):
    """
    HyperlinkedModelSerializer for SubRegion.
    """
    url = relations.HyperlinkedIdentityField(
        view_name='cities-light-api-subregion-detail')
    country = relations.HyperlinkedRelatedField(
        view_name='cities-light-api-country-detail', read_only=True)
    region = relations.HyperlinkedRelatedField(
        view_name='cities-light-api-region-detail', read_only=True)

    class Meta:
        model = SubRegion
        exclude = ('slug',)


class RegionSerializer(HyperlinkedModelSerializer):
    """
    HyperlinkedModelSerializer for Region.
    """
    url = relations.HyperlinkedIdentityField(
        view_name='cities-light-api-region-detail')
    country = relations.HyperlinkedRelatedField(
        view_name='cities-light-api-country-detail', read_only=True)

    class Meta:
        model = Region
        exclude = ('slug',)


class CountrySerializer(HyperlinkedModelSerializer):
    """
    HyperlinkedModelSerializer for Country.
    """
    url = relations.HyperlinkedIdentityField(
        view_name='cities-light-api-country-detail')

    class Meta:
        model = Country
        fields = '__all__'


class CitiesLightListModelViewSet(viewsets.ReadOnlyModelViewSet):
    def get_queryset(self):
        """
        Allows a GET param, 'q', to be used against name_ascii.
        """
        queryset = super().get_queryset()

        if self.request.GET.get('q', None):
            return queryset.filter(name_ascii__icontains=self.request.GET['q'])

        return queryset


class CountryModelViewSet(CitiesLightListModelViewSet):
    serializer_class = CountrySerializer
    queryset = Country.objects.all()


class RegionModelViewSet(CitiesLightListModelViewSet):
    serializer_class = RegionSerializer
    queryset = Region.objects.all()


class SubRegionModelViewSet(CitiesLightListModelViewSet):
    serializer_class = SubRegionSerializer
    queryset = SubRegion.objects.all()


class CityModelViewSet(CitiesLightListModelViewSet):
    """
    ListRetrieveView for City.
    """
    serializer_class = CitySerializer
    queryset = City.objects.all()

    def get_queryset(self):
        """
        Allows a GET param, 'q', to be used against search_names.
        """
        queryset = self.queryset

        if self.request.GET.get('q', None):
            return queryset.filter(
                search_names__icontains=self.request.GET['q'])

        return queryset


router = routers.SimpleRouter()
router.register(r'cities', CityModelViewSet, basename='cities-light-api-city')
router.register(r'countries', CountryModelViewSet,
                basename='cities-light-api-country')
router.register(r'regions', RegionModelViewSet,
                basename='cities-light-api-region')
router.register(r'subregions', SubRegionModelViewSet,
                basename='cities-light-api-subregion')

urlpatterns = [
    path('', include(router.urls)),
]



from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0006_compensate_for_0003_bytestring_bug'),
    ]

    operations = [
        migrations.AlterField(
            model_name='country',
            name='name',
            field=models.CharField(max_length=200, db_index=True),
        ),
    ]



"""."""
import os
from unittest import mock

from django import test
from django.core import management
from django.conf import settings


class FixtureDir:
    """Helper class to construct fixture paths."""

    def __init__(self, rel_path='', base_dir=None):
        """Class constructor.

        params:
        rel_path - subdir relative to base dir, e.g. 'aaaa/bbbb/'
        base_dir - base fixture directory (settings.FIXTURE_DIR by default)
        """
        self.base_dir = base_dir or settings.FIXTURE_DIR
        self.rel_path = rel_path

    def get_file_path(self, file_name):
        """
        Get full fixture path.

        Concatenate base_dir, rel_path and file_name.
        """
        return os.path.abspath(
            os.path.join(self.base_dir, self.rel_path, file_name)
        )


class TestImportBase(test.TransactionTestCase):
    """Base class for import testcases.

    Inherit from this class and use separate
    fixture subdirectory for each test_*.py.
    """

    maxDiff = 100000
    reset_sequences = True

    def import_data(self, srcdir, countries, regions, subregions, cities, trans, file_type="txt", **options):
        """Helper method to import Geonames data.

        Patch *_SOURCES settings and call 'cities_light' command with
        --force-import-all option.

        params:
        srcdir - source directory represented by FixtureDir object.
        countries - values for COUNTRY_SOURCES
        regions - values for REGION_SOURCES
        subregions - values for SUBREGION_SOURCES
        cities - values for CITY_SOURCES
        trans - values for TRANSLATION_SOURCES
        **options - passed to call_command() as is
        """

        def _s2l(param):
            return param if isinstance(param, list) else [param]

        def _patch(setting, *values):
            setting_to_patch = (
                    'cities_light.management.commands.cities_light.%s_SOURCES' %
                    setting.upper()
            )

            return mock.patch(
                setting_to_patch,
                ['file://%s.%s' % (srcdir.get_file_path(v), file_type) for v in values]
            )

        m_country = _patch('country', *_s2l(countries))
        m_region = _patch('region', *_s2l(regions))
        m_subregion = _patch('subregion', *_s2l(subregions))
        m_city = _patch('city', *_s2l(cities))
        m_tr = _patch('translation', *_s2l(trans))
        with m_country, m_region, m_subregion, m_city, m_tr:
            management.call_command('cities_light', progress=True,
                                    force_import_all=True,
                                    **options)



"""Tests for update records."""
import unittest

from dbdiff.fixture import Fixture
from .base import TestImportBase, FixtureDir


class TestUpdate(TestImportBase):
    """Tests update procedure."""

    def test_update_fields(self):
        """Test all fields are updated."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'initial_country',
            'initial_region',
            'initial_subregion',
            'initial_city',
            'initial_translations'
        )

        self.import_data(
            fixture_dir,
            'update_country',
            'update_region',
            'update_subregion',
            'update_city',
            'update_translations',
        )

        Fixture(
            fixture_dir.get_file_path('update_fields.json')
        ).assertNoDiff()

    def test_update_fields_wrong_timezone(self):
        """Test all fields are updated, but timezone field is wrong."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'initial_country',
            'initial_region',
            'initial_subregion',
            'initial_city',
            'initial_translations'
        )

        self.import_data(
            fixture_dir,
            'update_country',
            'update_region',
            'update_subregion',
            'update_city_wtz',
            'update_translations',
        )

        Fixture(
            fixture_dir.get_file_path('update_fields_wtz.json')
        ).assertNoDiff()

    def test_change_country(self):
        """Test change country for region/city."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'initial_country',
            'initial_region',
            'initial_subregion',
            'initial_city',
            'initial_translations'
        )

        self.import_data(
            fixture_dir,
            'change_country',
            'update_region',
            'update_subregion',
            'update_city',
            'update_translations',
        )

        Fixture(
            fixture_dir.get_file_path('change_country.json')
        ).assertNoDiff()

    def test_change_region_and_country(self):
        """Test change region and country."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'initial_country',
            'initial_region',
            'initial_subregion',
            'initial_city',
            'initial_translations'
        )

        self.import_data(
            fixture_dir,
            'change_country',
            'change_region',
            'update_subregion',
            'update_city',
            'update_translations',
        )

        Fixture(
            fixture_dir.get_file_path('change_region_and_country.json')
        ).assertNoDiff()

    def test_keep_slugs(self):
        """Test --keep-slugs option."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'initial_country',
            'initial_region',
            'initial_subregion',
            'initial_city',
            'initial_translations'
        )

        self.import_data(
            fixture_dir,
            'update_country',
            'update_region',
            'update_subregion',
            'update_city',
            'update_translations',
            keep_slugs=True
        )

        Fixture(
            fixture_dir.get_file_path('keep_slugs.json'),
        ).assertNoDiff()

    def test_add_records(self):
        """Test that new records are added."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'initial_country',
            'initial_region',
            'initial_subregion',
            'initial_city',
            'initial_translations'
        )

        self.import_data(
            fixture_dir,
            'add_country',
            'add_region',
            'add_subregion',
            'add_city',
            'add_translations'
        )

        Fixture(
            fixture_dir.get_file_path('add_records.json')
        ).assertNoDiff()

    def test_noinsert(self):
        """Test --noinsert option."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'initial_country',
            'initial_region',
            'initial_subregion',
            'initial_city',
            'initial_translations'
        )

        self.import_data(
            fixture_dir,
            'add_country',
            'add_region',
            'add_subregion',
            'add_city',
            'add_translations',
            noinsert=True
        )

        Fixture(
            fixture_dir.get_file_path('noinsert.json'),
        ).assertNoDiff()

    # TODO: make the test pass
    @unittest.skip("Obsolete records are not removed yet.")
    def test_remove_records(self):
        """Test that obsolete records are removed."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'remove_initial_country',
            'remove_initial_region',
            'remove_initial_subregion',
            'remove_initial_city',
            'remove_initial_translations'
        )

        self.import_data(
            fixture_dir,
            'remove_country',
            'remove_region',
            'remove_subregion',
            'remove_city',
            'remove_translations'
        )

        Fixture(
            fixture_dir.get_file_path('remove_records.json')
        ).assertNoDiff()



# Generated by Django 1.9.12 on 2017-03-13 07:37
import cities_light.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0007_make_country_name_not_unique'),
    ]

    operations = [
        migrations.AddField(
            model_name='city',
            name='timezone',
            field=models.CharField(blank=True, db_index=True, max_length=40, null=True, validators=[cities_light.validators.timezone_validator]),
        ),
    ]



from copy import copy

from django.contrib import admin
from django.contrib.admin.views.main import ChangeList

from .abstract_models import to_search
from . import forms
from .loading import get_cities_models

Country, Region, SubRegion, City = get_cities_models()


class CountryAdmin(admin.ModelAdmin):
    """
    ModelAdmin for Country.
    """

    list_display = (
        'name',
        'code2',
        'code3',
        'continent',
        'tld',
        'phone',
        'geoname_id',
    )
    search_fields = (
        'name',
        'name_ascii',
        'code2',
        'code3',
        'tld',
        'geoname_id',
    )
    list_filter = (
        'continent',
    )
    form = forms.CountryForm


admin.site.register(Country, CountryAdmin)


class RegionAdmin(admin.ModelAdmin):
    """
    ModelAdmin for Region.
    """
    list_filter = (
        'country__continent',
        'country',
    )
    search_fields = (
        'name',
        'name_ascii',
        'geoname_id',
    )
    list_display = (
        'name',
        'country',
        'geoname_id',
    )
    form = forms.RegionForm


admin.site.register(Region, RegionAdmin)


class SubRegionAdmin(admin.ModelAdmin):
    """
    ModelAdmin for SubRegion.
    """
    raw_id_fields = ["region"]
    list_filter = (
        'country__continent',
        'country',
        'region',
    )
    search_fields = (
        'name',
        'name_ascii',
        'geoname_id',
    )
    list_display = (
        'name',
        'country',
        'region',
        'geoname_id',
    )
    form = forms.SubRegionForm


admin.site.register(SubRegion, SubRegionAdmin)


class CityChangeList(ChangeList):
    def get_queryset(self, request):
        if 'q' in list(request.GET.keys()):
            request.GET = copy(request.GET)
            request.GET['q'] = to_search(request.GET['q'])
        return super().get_queryset(request)


class CityAdmin(admin.ModelAdmin):
    """
    ModelAdmin for City.
    """
    raw_id_fields = ["subregion", "region"]
    list_display = (
        'name',
        'subregion',
        'region',
        'country',
        'geoname_id',
        'timezone'
    )
    search_fields = (
        'search_names',
        'geoname_id',
        'timezone'
    )
    list_filter = (
        'country__continent',
        'country',
        'timezone'
    )
    form = forms.CityForm

    def get_changelist(self, request, **kwargs):
        return CityChangeList


admin.site.register(City, CityAdmin)



"""Management command to dump or load fixtures with geonames data."""

import os
import bz2
import logging
from argparse import RawTextHelpFormatter

try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

from django.db import transaction
from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError

from ...settings import DATA_DIR, FIXTURES_BASE_URL, CITIES_LIGHT_APP_NAME
from ...downloader import Downloader


class Command(BaseCommand):
    """Management command to dump or load fixtures with geonames data."""

    help = """
Dump or load fixtures with geonames data. Data dump is saved to
DATA_DIR/fixtures/, resulting in the following fixtures:

    cities_light_country.json.bz2
    cities_light_region.json.bz2
    cities_light_subregion.json.bz2
    cities_light_city.json.bz2

Loader by default uses the same DATA_DIR/fixtures/ folder (see
settings.CITIES_LIGHT_FIXTURES_BASE_URL), but you can customize its location
by specifying --base-url argument (do not forget the trailing slash):

    ./manage.py cities_light_fixtures load --base-url http://example.com/geo/

    or

    ./manage.py cities_light_fixtures load --base-url file:///tmp/folder/

It is possible to force fixture download by using the --force-fetch option:

    ./manage.py cities_light_fixtures load --force-fetch

It is possible to export using natural foreign keys by using the
--natural-foreign option (Take in consideration that this option will
going to take more time):

    ./manage.py cities_light_fixtures load --natural-foreign
    """.strip()

    logger = logging.getLogger('cities_light')

    COUNTRY_FIXTURE = 'cities_light_country.json.bz2'
    REGION_FIXTURE = 'cities_light_region.json.bz2'
    SUBREGION_FIXTURE = 'cities_light_subregion.json.bz2'
    CITY_FIXTURE = 'cities_light_city.json.bz2'

    def create_parser(self, *args, **kwargs):
        parser = super().create_parser(*args, **kwargs)
        parser.formatter_class = RawTextHelpFormatter
        return parser

    def add_arguments(self, parser):
        parser.add_argument(
            'subcommand',
            type=str,
            help='Subcommand (load/dump)'
        )
        parser.add_argument(
            '--natural-foreign',
            action='store_true',
            default=False,
            help='Export using natural foreign key'
        )
        parser.add_argument(
            '--force-fetch',
            action='store_true',
            default=False,
            help='Force fixture download'
        )
        parser.add_argument(
            '--base-url',
            action='store',
            metavar='BASE_URL',
            help='Base url to fetch from (default is '
                 'settings.CITIES_LIGHT_FIXTURES_BASE_URL)'
        )

    def handle(self, *args, **options):
        """Management command handler."""
        self.natural_foreign = options.get('natural_foreign')

        if not os.path.exists(DATA_DIR):
            self.logger.info('Creating %s', DATA_DIR)
            os.mkdir(DATA_DIR)

        fixtures_dir = os.path.join(DATA_DIR, 'fixtures')
        if not os.path.exists(fixtures_dir):
            self.logger.info('Creating %s', fixtures_dir)
            os.mkdir(fixtures_dir)

        self.country_path = os.path.join(fixtures_dir,
                                         self.COUNTRY_FIXTURE)
        self.region_path = os.path.join(fixtures_dir,
                                        self.REGION_FIXTURE)
        self.subregion_path = os.path.join(fixtures_dir,
                                           self.SUBREGION_FIXTURE)
        self.city_path = os.path.join(fixtures_dir,
                                      self.CITY_FIXTURE)

        subcommand = options.get('subcommand')

        if subcommand not in ('load', 'dump'):
            raise CommandError('Please specify either "load" '
                               'or "dump" command')

        if subcommand == 'load':
            base_url = options.get('base_url') or FIXTURES_BASE_URL
            if base_url is None:
                raise CommandError('Please specify --base-url or '
                                   'settings.CITIES_LIGHT_FIXTURES_BASE_URL')

            self.country_url = base_url + self.COUNTRY_FIXTURE
            self.region_url = base_url + self.REGION_FIXTURE
            self.subregion_url = base_url + self.SUBREGION_FIXTURE
            self.city_url = base_url + self.CITY_FIXTURE

            self.load_fixtures(**options)
        elif subcommand == 'dump':
            self.dump_fixtures()

    def dump_fixture(self, fixture, fixture_path,
                     natural_foreign: bool = False):
        """Dump single fixture."""
        self.logger.info('Dumping %s', fixture_path)

        out = StringIO()
        call_command('dumpdata',
                     fixture,
                     format="json",
                     natural_foreign=getattr(self, "natural_foreign",
                                             natural_foreign),
                     indent=1,
                     stdout=out)
        out.seek(0)
        with bz2.BZ2File(fixture_path, mode='w') as fixture_file:
            v = out.getvalue()
            fixture_file.write(v.encode())
        out.close()

    def dump_fixtures(self):
        """Dump Country/Region/City fixtures."""
        self.dump_fixture('{}.Country'.format(CITIES_LIGHT_APP_NAME),
                          self.country_path)
        self.dump_fixture('{}.Region'.format(CITIES_LIGHT_APP_NAME),
                          self.region_path)
        self.dump_fixture('{}.SubRegion'.format(CITIES_LIGHT_APP_NAME),
                          self.subregion_path)
        self.dump_fixture('{}.City'.format(CITIES_LIGHT_APP_NAME),
                          self.city_path)

    def load_fixture(self, source, destination, force=False):
        """Download and import single fixture."""
        downloader = Downloader()
        self.logger.info('Loading %s', source)
        downloader.download(source=source,
                            destination=destination,
                            force=force)
        call_command('loaddata', destination)

    @transaction.atomic
    def load_fixtures(self, **options):
        """Download and import Country/Region/City fixtures."""
        force = options.get('force_fetch')
        self.load_fixture(self.country_url, self.country_path, force=force)
        self.load_fixture(self.region_url, self.region_path, force=force)
        self.load_fixture(self.subregion_url, self.subregion_path, force=force)
        self.load_fixture(self.city_url, self.city_path, force=force)



import collections
import itertools
import os
import datetime
import logging
from argparse import RawTextHelpFormatter
import sys

if sys.platform != 'win32':
    import resource

try:
    import cPickle as pickle
except ImportError:
    import pickle

from django.conf import settings
from django.db import transaction, connection
from django.db import reset_queries, IntegrityError
from django.core.management.base import BaseCommand
from django.core.exceptions import ValidationError

import progressbar

from ...exceptions import *
from ...signals import *
from ...settings import *
from ...geonames import Geonames
from ...loading import get_cities_models
from ...validators import timezone_validator

Country, Region, SubRegion, City = get_cities_models()


class MemoryUsageWidget(progressbar.widgets.WidgetBase):
    def __call__(self, progress, data):
        if sys.platform == 'win32':
            return '?? MB'
        rusage = resource.getrusage(resource.RUSAGE_SELF)
        if sys.platform == 'darwin':
            return '%s MB' % (rusage.ru_maxrss // 1048576)
        else:
            return '%s MB' % (rusage.ru_maxrss // 1024)


class Command(BaseCommand):
    help = """
Download all files in CITIES_LIGHT_COUNTRY_SOURCES if they were updated or if
--force-all option was used.
Import country data if they were downloaded or if --force-import-all was used.

Same goes for CITIES_LIGHT_CITY_SOURCES.

It is possible to force the download of some files which have not been updated
on the server:

    manage.py cities_light --force cities15000 --force countryInfo

It is possible to force the import of files which weren't downloaded using the
--force-import option:

    manage.py cities_light --force-import cities15000 --force-import country
    """.strip()

    logger = logging.getLogger('cities_light')

    def create_parser(self, *args, **kwargs):
        parser = super().create_parser(*args, **kwargs)
        parser.formatter_class = RawTextHelpFormatter
        return parser

    def add_arguments(self, parser):
        parser.add_argument(
            '--force-import-all', action='store_true',
            default=False, help='Import even if files are up-to-date.'
        ),
        parser.add_argument(
            '--force-all', action='store_true', default=False,
            help='Download and import if files are up-to-date.'
        ),
        parser.add_argument(
            '--force-import', action='append', default=[],
            help='Import even if files matching files are up-to-date'
        ),
        parser.add_argument(
            '--force', action='append', default=[],
            help='Download and import even if matching files are up-to-date'
        ),
        parser.add_argument('--noinsert', action='store_true',
                            default=False,
                            help='Update existing data only'
                            ),
        parser.add_argument(
            '--hack-translations', action='store_true',
            default=False,
            help='Set this if you intend to import translations a lot'
        ),
        parser.add_argument(
            '--keep-slugs', action='store_true',
            default=False,
            help='Do not update slugs'
        ),
        parser.add_argument('--progress', action='store_true',
                            default=False,
                            help='Show progress bar'
                            ),

    def progress_init(self):
        """Initialize progress bar."""
        if self.progress_enabled:
            self.progress_widgets = [
                'RAM used: ',
                MemoryUsageWidget(),
                ' ',
                progressbar.ETA(),
                ' Done: ',
                progressbar.Percentage(),
                progressbar.Bar(),
            ]

    def progress_start(self, max_value):
        """Start progress bar."""
        if self.progress_enabled:
            self.progress = progressbar.ProgressBar(
                max_value=max_value,
                widgets=self.progress_widgets
            ).start()

    def progress_update(self, value):
        """Update progress bar."""
        if self.progress_enabled:
            self.progress.update(value)

    def progress_finish(self):
        """Finalize progress bar."""
        if self.progress_enabled:
            self.progress.finish()

    def handle(self, *args, **options):
        # initialize lazy identity maps
        self._clear_identity_maps()

        if not os.path.exists(DATA_DIR):
            self.logger.info('Creating %s', DATA_DIR)
            os.mkdir(DATA_DIR)

        install_file_path = os.path.join(DATA_DIR, 'install_datetime')
        translation_hack_path = os.path.join(DATA_DIR, 'translation_hack')

        self.noinsert = options.get('noinsert', False)
        self.keep_slugs = options.get('keep_slugs', False)
        self.progress_enabled = options.get('progress')

        self.progress_init()

        sources = list(itertools.chain(
            COUNTRY_SOURCES,
            REGION_SOURCES,
            SUBREGION_SOURCES,
            CITY_SOURCES,
            TRANSLATION_SOURCES,
        ))

        for url in sources:
            if url in TRANSLATION_SOURCES:
                # free some memory
                self._clear_identity_maps()

            destination_file_name = url.split('/')[-1]

            force = options.get('force_all', False)
            if not force:
                for f in options['force']:
                    if f in destination_file_name or f in url:
                        force = True

            geonames = Geonames(url, force=force)
            downloaded = geonames.downloaded

            force_import = options.get('force_import_all', False)

            if not force_import:
                for f in options['force_import']:
                    if f in destination_file_name or f in url:
                        force_import = True

            if not os.path.exists(install_file_path):
                self.logger.info(
                    'Forced import of %s because data do not seem'
                    ' to have installed successfuly yet, note that this is'
                    ' equivalent to --force-import-all.',
                    destination_file_name)
                force_import = True

            if downloaded or force_import:
                self.logger.info('Importing %s', destination_file_name)

                if url in TRANSLATION_SOURCES:
                    if options.get('hack_translations', False):
                        if os.path.exists(translation_hack_path):
                            self.logger.debug(
                                'Using translation parsed data: %s',
                                translation_hack_path)
                            continue

                i = 0
                self.progress_start(geonames.num_lines())

                for items in geonames.parse():
                    if url in CITY_SOURCES:
                        self.city_import(items)
                    elif url in REGION_SOURCES:
                        self.region_import(items)
                    elif url in COUNTRY_SOURCES:
                        self.country_import(items)
                    elif url in SUBREGION_SOURCES:
                        self.subregion_import(items)
                    elif url in TRANSLATION_SOURCES:
                        self.translation_parse(items)

                    # prevent memory leaks in DEBUG mode
                    # https://docs.djangoproject.com/en/1.9/faq/models/
                    # #how-can-i-see-the-raw-sql-queries-django-is-running
                    if settings.DEBUG:
                        reset_queries()

                    i += 1
                    self.progress_update(i)

                self.progress_finish()

                if url in TRANSLATION_SOURCES and options.get(
                        'hack_translations', False):
                    with open(translation_hack_path, 'w+') as f:
                        pickle.dump(self.translation_data, f)

        if options.get('hack_translations', False):
            with open(translation_hack_path, 'r') as f:
                self.translation_data = pickle.load(f)

        self.logger.info('Importing parsed translation in the database')
        self.translation_import()

        with open(install_file_path, 'wb+') as f:
            pickle.dump(datetime.datetime.now(), f)

    def _clear_identity_maps(self):
        """Clear identity maps and free some memory."""
        if getattr(self, '_country_codes', False):
            del self._country_codes
        if getattr(self, '_region_codes', False):
            del self._region_codes
        if getattr(self, '_subregion_codes', False):
            del self._subregion_codes
        self._country_codes = {}
        self._region_codes = collections.defaultdict(dict)
        self._subregion_codes = collections.defaultdict(
            lambda: collections.defaultdict(dict))

    def _get_country_id(self, country_code2):
        """
        Simple lazy identity map for code2->country
        """
        if country_code2 not in self._country_codes:
            self._country_codes[country_code2] = \
                Country.objects.get(code2=country_code2).pk

        return self._country_codes[country_code2]

    def _get_region_id(self, country_code2, region_id):
        """
        Simple lazy identity map for (country_code2, region_id)->region
        """
        country_id = self._get_country_id(country_code2)
        if region_id not in self._region_codes[country_id]:
            self._region_codes[country_id][region_id] = Region.objects.get(
                country_id=country_id, geoname_code=region_id).pk

        return self._region_codes[country_id][region_id]

    def _get_subregion_id(self, country_code2, region_id, subregion_id):
        """
        Simple lazy identity map for (country_code2, region_id,
        subregion_id)->subregion
        """
        country_id = self._get_country_id(country_code2)
        if region_id not in self._region_codes[country_id]:
            self._region_codes[country_id][region_id] = Region.objects.get(
                country_id=country_id, geoname_code=region_id).pk

        if subregion_id not in self._subregion_codes[country_id][region_id]:
            self._subregion_codes[country_id][region_id][subregion_id] = \
                SubRegion.objects.get(
                    region_id=self._region_codes[country_id][region_id],
                    geoname_code=subregion_id).pk
        return self._subregion_codes[country_id][region_id][subregion_id]

    def country_import(self, items):
        try:
            country_items_pre_import.send(sender=self, items=items)
        except InvalidItems:
            return
        force_insert = False
        force_update = False
        if items[ICountry.geonameid] == '':
            return
        try:
            country = Country.objects.get(geoname_id=items[ICountry.geonameid])
            force_update = True
        except Country.DoesNotExist:
            if self.noinsert:
                return
            country = Country(geoname_id=items[ICountry.geonameid])
            force_insert = True

        country.name = items[ICountry.name]
        country.code2 = items[ICountry.code2]
        country.code3 = items[ICountry.code3]
        country.continent = items[ICountry.continent]
        country.tld = items[ICountry.tld][1:]  # strip the leading dot
        # Strip + prefix for consistency. Note that some countries have several
        # prefixes ie. Puerto Rico
        country.phone = items[ICountry.phone].replace('+', '')
        # Clear name_ascii to always update it by set_name_ascii() signal
        country.name_ascii = ''

        if force_update and not self.keep_slugs:
            country.slug = None

        country_items_post_import.send(
            sender=self,
            instance=country,
            items=items
        )

        self.save(
            country,
            force_insert=force_insert,
            force_update=force_update
        )

    def region_import(self, items):
        try:
            region_items_pre_import.send(sender=self, items=items)
        except InvalidItems:
            return

        force_insert = False
        force_update = False
        try:
            region = Region.objects.get(geoname_id=items[IRegion.geonameid])
            force_update = True
        except Region.DoesNotExist:
            if self.noinsert:
                return
            region = Region(geoname_id=items[IRegion.geonameid])
            force_insert = True

        name = items[IRegion.name]
        if not items[IRegion.name]:
            name = items[IRegion.asciiName]

        code2, geoname_code = items[IRegion.code].split('.')
        country_id = self._get_country_id(code2)

        save = False
        if region.name != name:
            region.name = name
            save = True

        if region.country_id != country_id:
            region.country_id = country_id
            save = True

        if region.geoname_code != geoname_code:
            region.geoname_code = geoname_code
            save = True

        if region.name_ascii != items[IRegion.asciiName]:
            region.name_ascii = items[IRegion.asciiName]
            save = True

        if force_update and not self.keep_slugs:
            region.slug = None

        region_items_post_import.send(
            sender=self,
            instance=region,
            items=items
        )

        if save:
            self.save(
                region,
                force_insert=force_insert,
                force_update=force_update
            )

    def subregion_import(self, items):
        try:
            subregion_items_pre_import.send(sender=self, items=items)
        except InvalidItems:
            return

        force_insert = force_update = False
        try:
            subregion = SubRegion.objects.filter(
                geoname_id=items[ISubRegion.geonameid]).first()
            if subregion:
                force_update = True
            elif not subregion and self.noinsert:
                return
            else:
                subregion = SubRegion(geoname_id=items[ISubRegion.geonameid])
                force_insert = True
        except SubRegion.DoesNotExist:
            if self.noinsert:
                return
            subregion = SubRegion(geoname_id=items[ISubRegion.geonameid])
            force_insert = True

        name = items[ISubRegion.name]
        if not items[ISubRegion.name]:
            name = items[ISubRegion.asciiName]

        code2, admin1Code, geoname_code = items[ISubRegion.code].split('.')
        try:
            country_id = self._get_country_id(code2)
        except Country.DoesNotExist:
            country_id = None

        try:
            region_id = self._get_region_id(
                code2,
                admin1Code
            )
        except Region.DoesNotExist:
            region_id = None

        save = False
        if subregion.name != name:
            subregion.name = name
            save = True

        if subregion.country_id != country_id:
            subregion.country_id = country_id
            save = True

        if subregion.region_id != region_id:
            subregion.region_id = region_id
            save = True

        if subregion.geoname_code != geoname_code:
            subregion.geoname_code = geoname_code
            save = True

        if subregion.name_ascii != items[ISubRegion.asciiName]:
            subregion.name_ascii = items[ISubRegion.asciiName]
            save = True

        if force_update and not self.keep_slugs:
            subregion.slug = None

        subregion_items_post_import.send(
            sender=self,
            instance=subregion,
            items=items
        )

        if save:
            self.save(
                subregion,
                force_insert=force_insert,
                force_update=force_update
            )

    def city_import(self, items):
        try:
            city_items_pre_import.send(sender=self, items=items)
        except InvalidItems:
            return

        force_insert = False
        force_update = False
        try:
            city = City.objects.get(geoname_id=items[ICity.geonameid])
            force_update = True
        except City.DoesNotExist:
            if self.noinsert:
                return
            city = City(geoname_id=items[ICity.geonameid])
            force_insert = True

        try:
            country_id = self._get_country_id(items[ICity.countryCode])
        except Country.DoesNotExist:
            if self.noinsert:
                return
            else:
                raise

        try:
            region_id = self._get_region_id(
                items[ICity.countryCode],
                items[ICity.admin1Code]
            )
        except Region.DoesNotExist:
            region_id = None

        try:
            subregion_id = self._get_subregion_id(
                items[ICity.countryCode],
                items[ICity.admin1Code],
                items[ICity.admin2Code]
            )
        except (SubRegion.DoesNotExist, Region.DoesNotExist):
            subregion_id = None

        save = False
        if city.country_id != country_id:
            city.country_id = country_id
            save = True

        if city.region_id != region_id:
            city.region_id = region_id
            save = True

        if city.subregion_id != subregion_id:
            city.subregion_id = subregion_id
            save = True

        if city.name != items[ICity.name]:
            city.name = items[ICity.name]
            save = True

        if city.name_ascii != items[ICity.asciiName]:
            # useful for cities with chinese names
            city.name_ascii = items[ICity.asciiName]
            save = True

        if city.latitude != items[ICity.latitude]:
            city.latitude = items[ICity.latitude]
            save = True

        if city.longitude != items[ICity.longitude]:
            city.longitude = items[ICity.longitude]
            save = True

        if city.population != items[ICity.population]:
            city.population = items[ICity.population]
            save = True

        if city.feature_code != items[ICity.featureCode]:
            city.feature_code = items[ICity.featureCode]
            save = True

        if city.timezone != items[ICity.timezone]:
            try:
                timezone_validator(items[ICity.timezone])
                city.timezone = items[ICity.timezone]
            except ValidationError as e:
                city.timezone = None
                self.logger.warning(e.messages)
            save = True

        altnames = items[ICity.alternateNames]
        if not TRANSLATION_SOURCES and city.alternate_names != altnames:
            city.alternate_names = altnames
            save = True

        if force_update and not self.keep_slugs:
            city.slug = None

        city_items_post_import.send(
            sender=self,
            instance=city,
            items=items,
            save=save
        )

        if save:
            self.save(
                city,
                force_insert=force_insert,
                force_update=force_update
            )

    def translation_parse(self, items):
        if not hasattr(self, 'translation_data'):
            self.country_ids = set(Country.objects.values_list(
                'geoname_id', flat=True))
            self.region_ids = set(Region.objects.values_list(
                'geoname_id', flat=True))
            self.city_ids = set(City.objects.values_list(
                'geoname_id', flat=True))

            self.translation_data = collections.OrderedDict((
                (Country, {}),
                (Region, {}),
                (City, {}),
            ))

        # https://code.djangoproject.com/ticket/21597#comment:29
        # https://github.com/yourlabs/django-cities-light/commit/e7f69af01760c450b4a72db84fda3d98d6731928
        if 'mysql' in settings.DATABASES['default']['ENGINE']:
            connection.close()

        try:
            translation_items_pre_import.send(sender=self, items=items)
        except InvalidItems:
            return

        if len(items) > 5:
            # avoid shortnames, colloquial, and historic
            return

        item_lang = items[IAlternate.language]

        if item_lang not in TRANSLATION_LANGUAGES:
            return

        item_geoid = items[IAlternate.geonameid]
        item_name = items[IAlternate.name]

        # arg optimisation code kills me !!!
        item_geoid = int(item_geoid)

        if item_geoid in self.country_ids:
            model_class = Country
        elif item_geoid in self.region_ids:
            model_class = Region
        elif item_geoid in self.city_ids:
            model_class = City
        else:
            return

        if item_geoid not in self.translation_data[model_class]:
            self.translation_data[model_class][item_geoid] = {}

        if item_lang not in self.translation_data[model_class][item_geoid]:
            self.translation_data[model_class][item_geoid][item_lang] = []

        self.translation_data[model_class][item_geoid][item_lang].append(
            item_name)

    def translation_import(self):
        data = getattr(self, 'translation_data', None)

        if not data:
            return

        max = 0
        for model_class, model_class_data in data.items():
            max += len(model_class_data.keys())

        i = 0
        self.progress_start(max)

        for model_class, model_class_data in data.items():
            for geoname_id, geoname_data in model_class_data.items():
                try:
                    model = model_class.objects.get(geoname_id=geoname_id)
                except model_class.DoesNotExist:
                    continue

                save = False
                alternate_names = set()
                for lang, names in geoname_data.items():
                    if lang == 'post':
                        # we might want to save the postal codes somewhere
                        # here's where it will all start ...
                        continue

                    for name in names:
                        if name == model.name:
                            continue

                        alternate_names.add(name)

                alternate_names = ';'.join(sorted(alternate_names))
                if model.alternate_names != alternate_names:
                    model.alternate_names = alternate_names
                    save = True

                if save:
                    model.save(force_update=True)

                i += 1
                self.progress_update(i)

        self.progress_finish()

    def save(self, model, force_insert=False, force_update=False):
        try:
            with transaction.atomic():
                self.logger.debug('Saving %s', model.name)
                model.save(
                    force_insert=force_insert,
                    force_update=force_update
                )
        except IntegrityError as e:
            # Regarding %r see the https://code.djangoproject.com/ticket/20572
            # Also related to http://bugs.python.org/issue2517
            self.logger.warning('Saving %s failed: %r', model, e)



from django import test
from django.apps import apps
from django.db.migrations.autodetector import MigrationAutodetector
from django.db.migrations.loader import MigrationLoader
from django.db.migrations.questioner import (
    InteractiveMigrationQuestioner, )
from django.db.migrations.state import ProjectState


class TestNoMigrationLeft(test.TestCase):
    def test_no_migration_left(self):
        loader = MigrationLoader(None, ignore_no_migrations=True)
        conflicts = loader.detect_conflicts()
        app_labels = ['cities_light']

        autodetector = MigrationAutodetector(
            loader.project_state(),
            ProjectState.from_apps(apps),
            InteractiveMigrationQuestioner(specified_apps=app_labels, dry_run=True),
        )

        changes = autodetector.changes(
            graph=loader.graph,
            trim_to_apps=app_labels or None,
            convert_apps=app_labels or None,
        )

        assert 'cities_light' not in changes



# Generated by Django 4.0.5 on 2022-06-10 22:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0010_auto_20200508_1851'),
    ]

    operations = [
        migrations.AlterField(
            model_name='city',
            name='country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='cities_light.country'),
        ),
        migrations.AlterField(
            model_name='city',
            name='region',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='cities_light.region'),
        ),
        migrations.AlterField(
            model_name='region',
            name='country',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='cities_light.country'),
        ),
    ]



"""
By default, all models are taken from this package.
But it is possible to customise these models to add some fields.
For such purpose cities_light models are defined as abstract (without
customisation they all inherit abstract versions automatically
without changes).

Steps to customise cities_light models
======================================

- Define **all** of cities abstract models in your app:
    .. code:: python

        # yourapp/models.py

        from cities_light.abstract_models import (AbstractCity, AbstractRegion,
            AbstractSubRegion, AbstractCountry)
        from cities_light.receivers import connect_default_signals


        class Country(AbstractCountry):
            pass
        connect_default_signals(Country)

        class Region(AbstractRegion):
            pass
        connect_default_signals(Region)

        class SubRegion(AbstractSubRegion):
            pass
        connect_default_signals(SubRegion)

        class City(AbstractCity):
            modification_date = models.CharField(max_length=40)
        connect_default_signals(City)

- Add post import processing to you model *[optional]*:
    .. code:: python

        import cities_light
        from cities_light.settings import ICity

        def set_city_fields(sender, instance, items, **kwargs):
            instance.modification_date = items[ICity.modificationDate]
        cities_light.signals.city_items_post_import.connect(set_city_fields)

- Define settings.py:
    .. code:: python

        INSTALLED_APPS = [
            # ...
            'cities_light',
            'yourapp',
        ]

        CITIES_LIGHT_APP_NAME = 'yourapp'

        # Disable built-in cities_light migrations
        MIGRATION_MODULES = {
            'cities_light': None
        }

- Create your own migrations:
    .. code::

        python manage.py makemigrations yourapp
        python manage.py migrate

That's all!

**Notes**:
    - model names can't be modified, i.e. you have to use exactly
      City, Country, Region names and not MyCity, MyCountry, MyRegion.
    - Connect default signals for every custom model by calling
      ``connect_default_signals`` (or not, if you don't want to trigger
      default signals).
    - if in further versions of cities_light abstract models will be
      updated (some fields will be added/removed), you have to deal with
      migrations by yourself, as models are on your own now.
"""

# some imports are present for backwards compatibility and migration process
from .abstract_models import (AbstractCountry, AbstractRegion,
                              AbstractSubRegion, AbstractCity,
                              CONTINENT_CHOICES,
                              ToSearchTextField, to_search, to_ascii)

from .signals import *
from .receivers import *
from .settings import *

__all__ = ['CONTINENT_CHOICES', 'to_search', 'to_ascii', 'filter_non_cities',
           'filter_non_included_countries_country',
           'filter_non_included_countries_region',
           'filter_non_included_countries_subregion',
           'filter_non_included_countries_city']

if CITIES_LIGHT_APP_NAME == DEFAULT_APP_NAME:
    class Country(AbstractCountry):
        pass
    connect_default_signals(Country)

    __all__.append('Country')

    class Region(AbstractRegion):
        pass
    connect_default_signals(Region)

    __all__.append('Region')

    class SubRegion(AbstractSubRegion):
        pass
    connect_default_signals(SubRegion)

    __all__.append('SubRegion')

    class City(AbstractCity):
        pass
    connect_default_signals(City)

    __all__.append('City')



from django.db import models, migrations
import cities_light.models

from cities_light.settings import INDEX_SEARCH_NAMES


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0002_city'),
    ]

    operations = [
        migrations.AlterField(
            model_name='city',
            name='search_names',
            field=cities_light.models.ToSearchTextField(default=b'', max_length=4000, db_index=INDEX_SEARCH_NAMES, blank=True),
            preserve_default=True,
        ),
    ]



"""Basic tests for contrib modules."""
import json

from django.test.utils import override_settings
from django.test.client import Client

from .base import TestImportBase, FixtureDir
from ..contrib.ajax_selects_lookups import (
    CountryLookup,
    RegionLookup,
    CityLookup
)


class TestRestFramework(TestImportBase):
    """Tests for django-rest-framework API endpoints."""

    longMessage = True

    json_headers = {
        'HTTP_ACCEPT': 'application/json'
    }

    def setUp(self):
        """Test init."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'add_country',
            'add_region',
            'add_subregion',
            'add_city',
            'add_translations',
        )

        self.json_client = Client(**self.json_headers)

    def json_get(self, url):
        """Make GET request and assert that status is 200."""
        response = self.json_client.get(url)
        self.assertEqual(
            response.status_code, 200,
            msg='Status code should be 200'
        )
        return json.loads(response.content.decode('utf-8'))

    @override_settings(ROOT_URLCONF='cities_light.contrib.restframework3')
    def test_get_countries(self):
        """Test that full list of countries can be retrieved."""
        data = self.json_get('/countries/')
        self.assertEqual(len(data), 3, msg='Should retrieve 3 countries')
        names = [i['name_ascii'] for i in data]
        self.assertIn('Russia', names)
        self.assertIn('United Kingdom', names)
        self.assertIn('USSR', names)

    @override_settings(ROOT_URLCONF='cities_light.contrib.restframework3')
    def test_search_countries(self):
        """Test that search works for the list of countries."""
        data = self.json_get('/countries/?q=Gondor')
        self.assertEqual(len(data), 0, msg='Should find 0 countries')

        data = self.json_get('/countries/?q=SR')
        self.assertEqual(len(data), 1, msg='Should find 1 country')
        self.assertEqual(data[0]['name_ascii'], 'USSR')

    @override_settings(ROOT_URLCONF='cities_light.contrib.restframework3')
    def test_get_regions(self):
        """Test that full list of regions can be retrieved."""
        data = self.json_get('/regions/')
        self.assertEqual(len(data), 3, msg='Should retrieve 3 regions')
        self.assertEqual(data[0]['name_ascii'], 'Kemerovo')
        self.assertEqual(data[1]['name_ascii'], 'Kuzbass')
        self.assertEqual(data[2]['name_ascii'], 'Scotland')

    @override_settings(ROOT_URLCONF='cities_light.contrib.restframework3')
    def test_search_regions(self):
        """Test that search works for the list of regions."""
        data = self.json_get('/regions/?q=Anórien')
        self.assertEqual(len(data), 0, msg='Should find 0 regions')

        data = self.json_get('/regions/?q=uzB')
        self.assertEqual(len(data), 1, msg='Should find 1 region')
        self.assertEqual(data[0]['name_ascii'], 'Kuzbass')

    @override_settings(ROOT_URLCONF='cities_light.contrib.restframework3')
    def test_get_cities(self):
        """Test that full list of cities can be retrieved."""
        data = self.json_get('/cities/')
        self.assertEqual(len(data), 5, msg='Should retrieve 5 cities')
        self.assertEqual(data[0]['name_ascii'], 'Belovo')
        self.assertEqual(data[1]['name_ascii'], 'Kemerovo')
        self.assertEqual(data[2]['name_ascii'], 'Kiselevsk')
        self.assertEqual(data[3]['name_ascii'], 'Nedd')
        self.assertEqual(data[4]['name_ascii'], 'Novokuznetsk')

    @override_settings(ROOT_URLCONF='cities_light.contrib.restframework3')
    def test_search_cities(self):
        """Test that search works for the list of cities."""
        data = self.json_get('/cities/?q=Minas')
        self.assertEqual(len(data), 0, msg='Should find 0 cities')

        data = self.json_get('/cities/?q=ussr')
        self.assertEqual(len(data), 2, msg='Should find 2 cities')
        self.assertEqual(data[0]['name_ascii'], 'Belovo')
        self.assertEqual(data[1]['name_ascii'], 'Kiselevsk')


class TestAjaxSelectsLookups(TestImportBase):
    """Tests for ajax selects lookups."""

    longMessage = True

    def setUp(self):
        """Test init."""
        fixture_dir = FixtureDir('update')

        self.import_data(
            fixture_dir,
            'add_country',
            'add_region',
            'add_subregion',
            'add_city',
            'add_translations',
        )

    def test_country_lookup(self):
        """Test for CountryLookup.get_query method."""
        country_lookup = CountryLookup()
        lookup_query = country_lookup.get_query(q='Valinor', request=None)
        self.assertFalse(lookup_query.exists(), msg='Country should not exist')
        lookup_query = country_lookup.get_query(q='Russia', request=None)
        self.assertEqual(len(lookup_query), 1, msg='1 country should be found')
        self.assertEqual(lookup_query[0].name, 'Russia')
        lookup_query = country_lookup.get_query(q='sr', request=None)
        self.assertEqual(len(lookup_query), 1, msg='1 country should be found')
        self.assertEqual(lookup_query[0].name, 'USSR')
        lookup_query = country_lookup.get_query(q='us', request=None)
        self.assertEqual(
            len(lookup_query), 2,
            msg='2 countries should be found'
        )
        self.assertEqual(lookup_query[0].name, 'Russia')
        self.assertEqual(lookup_query[1].name, 'USSR')

    def test_region_lookup(self):
        """Test for RegionLookup.get_query method."""
        region_lookup = RegionLookup()
        lookup_query = region_lookup.get_query(q='Morthond Vale', request=None)
        self.assertFalse(lookup_query.exists(), msg='Region should not exist')
        lookup_query = region_lookup.get_query(q='Kuzbass', request=None)
        self.assertEqual(len(lookup_query), 1, msg='1 region should be found')
        self.assertEqual(lookup_query[0].name, 'Kuzbass')
        lookup_query = region_lookup.get_query(q='emer', request=None)
        self.assertEqual(len(lookup_query), 1, msg='1 region should be found')
        self.assertEqual(lookup_query[0].name, 'Kemerovo')
        lookup_query = region_lookup.get_query(q='K', request=None)
        self.assertEqual(
            len(lookup_query), 2,
            msg='2 regions should be found'
        )
        self.assertEqual(lookup_query[0].name, 'Kemerovo')
        self.assertEqual(lookup_query[1].name, 'Kuzbass')

    def test_city_lookup(self):
        """Test for CityLookup.get_query method."""
        city_lookup = CityLookup()
        lookup_query = city_lookup.get_query(q='Minas Morgul', request=None)
        self.assertFalse(lookup_query.exists(), msg='City should not exist')
        lookup_query = city_lookup.get_query(q='Novokuznetsk', request=None)
        self.assertEqual(len(lookup_query), 1, msg='1 city should be found')
        self.assertEqual(lookup_query[0].name, 'Novokuznetsk')
        lookup_query = city_lookup.get_query(q='ovo', request=None)
        self.assertEqual(len(lookup_query), 3, msg='3 cities should be found')
        self.assertEqual(lookup_query[0].name, 'Belovo')
        self.assertEqual(lookup_query[1].name, 'Kemerovo')
        self.assertEqual(lookup_query[2].name, 'Novokuznetsk')



"""Test for models methods."""
from django import test
from django.conf import settings
from django.core.exceptions import ValidationError

from ..loading import get_cities_model
from ..validators import timezone_validator


class TestModels(test.TransactionTestCase):
    """Tests for cities light models methods."""

    longMessage = True

    def test_city_get_timezone(self):
        """Test City.get_timezone_info method."""
        city_model = get_cities_model('City')

        city = city_model(
            geoname_id='123456',
            name='city',
            timezone='Asia/Novosibirsk'
        )
        self.assertEqual(city.get_timezone_info().zone, 'Asia/Novosibirsk')

        city = city_model(
            geoname_id='123457',
            name='city',
            timezone='Mars/Cidonia'
        )
        self.assertEqual(city.get_timezone_info().zone, settings.TIME_ZONE)

        city = city_model(
            geoname_id='123457',
            name='city',
            timezone=None
        )
        self.assertEqual(city.get_timezone_info().zone, settings.TIME_ZONE)

        city = city_model(
            geoname_id='123457',
            name='city',
            timezone=''
        )
        self.assertEqual(city.get_timezone_info().zone, settings.TIME_ZONE)

    def test_timezone_validator(self):
        """Test timezone_validator."""
        with self.assertRaises(ValidationError) as e:
            timezone_validator(None)
        self.assertEqual(
            e.exception.messages[0], 'Timezone validation error: None')
        self.assertEqual(e.exception.code, 'timezone_error')

        with self.assertRaises(ValidationError) as e:
            timezone_validator('')
        self.assertEqual(
            e.exception.messages[0], 'Timezone validation error: ')
        self.assertEqual(e.exception.code, 'timezone_error')

        with self.assertRaises(ValidationError) as e:
            timezone_validator('Mars/Cidonia')
        self.assertEqual(
            e.exception.messages[0], 'Timezone validation error: Mars/Cidonia')
        self.assertEqual(e.exception.code, 'timezone_error')

        try:
            value = 'Asia/Novokuznetsk'
            timezone_validator(value)
        except Exception:
            self.assertTrue(
                False,
                'Error call timezone_validator with true value: %s' % value)

    def test_city_search_names_icontains(self):
        """Test for City.search_names space-insensitive lookup."""
        country_model = get_cities_model('Country')
        region_model = get_cities_model('Region')
        subregion_model = get_cities_model('SubRegion')
        city_model = get_cities_model('City')

        country = country_model(
            name='Country',
            name_ascii='Country',
            geoname_id='123456',
            continent='EU')
        country.save()

        region = region_model(
            name='Region',
            name_ascii='Region',
            geoname_id='123457',
            display_name='Region',
            country=country
        )
        region.save()

        subregion = subregion_model(
            name='SubRegion',
            name_ascii='SubRegion',
            geoname_id='987654',
            display_name='SubRegion',
            region=region,
            country=country
        )
        subregion.save()

        city1 = city_model(
            name='First City',
            name_ascii='First City',
            geoname_id='123458',
            display_name='First City',
            search_names='firstcityregioncountry',
            region=region,
            country=country,
            subregion=subregion
        )
        city1.save()

        city2 = city_model(
            name='Second City',
            name_ascii='Second City',
            geoname_id='123459',
            display_name='Second City',
            search_names='secondcityregioncountry',
            region=region,
            country=country,
            subregion=subregion
        )
        city2.save()

        city_qs = city_model.objects.filter(
            search_names__icontains='First City'
        )

        self.assertEqual(len(city_qs), 1, msg='Should find 1 city')
        self.assertEqual(city_qs[0].name, city1.name)

        city_qs = city_model.objects.filter(
            search_names__icontains='Second City'
        )

        self.assertEqual(len(city_qs), 1, msg='Should find 1 city')
        self.assertEqual(city_qs[0].name, city2.name)

        city_qs = city_model.objects.filter(
            search_names__icontains='Third City'
        )
        self.assertEqual(len(city_qs), 0, msg='Should find 0 cities')

        city_qs = city_model.objects.filter(
            search_names__icontains='Region Country'
        )
        self.assertEqual(len(city_qs), 2, msg='Should find 2 cities')
        self.assertEqual(city_qs[0].name, city1.name)
        self.assertEqual(city_qs[1].name, city2.name)

        city_qs = city_model.objects.filter(
            search_names__icontains='gion coun'
        )
        self.assertEqual(len(city_qs), 2, msg='Should find 2 cities')
        self.assertEqual(city_qs[0].name, city1.name)
        self.assertEqual(city_qs[1].name, city2.name)



from django.db import models, migrations
import autoslug.fields


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0003_auto_20141120_0342'),
    ]

    operations = [
        migrations.AlterField(
            model_name='city',
            name='slug',
            field=autoslug.fields.AutoSlugField(populate_from='name_ascii', editable=False),
        ),
        migrations.AlterField(
            model_name='country',
            name='slug',
            field=autoslug.fields.AutoSlugField(populate_from='name_ascii', editable=False),
        ),
        migrations.AlterField(
            model_name='region',
            name='slug',
            field=autoslug.fields.AutoSlugField(populate_from='name_ascii', editable=False),
        ),
    ]



from setuptools import setup, find_packages
import os
import os.path


# Utility function to read the README file.
# Used for the long_description. It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setup(
    name="django-cities-light",
    use_scm_version={
        "version_scheme": "post-release",
        "write_to": "src/cities_light/version.py",
    },
    setup_requires=["setuptools_scm"],
    description="Simple alternative to django-cities",
    author="James Pic,Dominick Rivard,Alexey Evseev",
    author_email="jamespic@gmail.com, dominick.rivard@gmail.com, myhappydo@gmail.com",
    url="https://github.com/yourlabs/django-cities-light",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    include_package_data=True,
    zip_safe=False,
    long_description=read("README.rst"),
    license="MIT",
    keywords="django cities countries postal codes",
    install_requires=[
        "pytz",
        "unidecode>=0.04.13",
        "django-autoslug>=1.9.8",
        "progressbar2>=3.51.4",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



from django.db import models, migrations
import autoslug.fields
import cities_light.models
from cities_light.settings import INDEX_SEARCH_NAMES


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='City',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name_ascii', models.CharField(db_index=True, max_length=200, blank=True)),
                ('slug', autoslug.fields.AutoSlugField(editable=False)),
                ('geoname_id', models.IntegerField(unique=True, null=True, blank=True)),
                ('alternate_names', models.TextField(default='', null=True, blank=True)),
                ('name', models.CharField(max_length=200, db_index=True)),
                ('display_name', models.CharField(max_length=200)),
                ('search_names', models.TextField(default='', max_length=4000, db_index=INDEX_SEARCH_NAMES, blank=True)),
                ('latitude', models.DecimalField(null=True, max_digits=8, decimal_places=5, blank=True)),
                ('longitude', models.DecimalField(null=True, max_digits=8, decimal_places=5, blank=True)),
                ('region', models.ForeignKey(to_field='id', blank=True, to='cities_light.Region', null=True, on_delete=models.CASCADE)),
                ('country', models.ForeignKey(to='cities_light.Country', to_field='id', on_delete=models.CASCADE)),
                ('population', models.BigIntegerField(db_index=True, null=True, blank=True)),
                ('feature_code', models.CharField(db_index=True, max_length=10, null=True, blank=True)),
            ],
            options={
                'ordering': ['name'],
                'unique_together': set([('region', 'name'), ('region', 'slug')]),
                'abstract': False,
                'verbose_name_plural': 'cities',
            },
            bases=(models.Model,),
        ),
    ]



import warnings

import unidecode

from django.utils.encoding import force_str

from ..abstract_models import to_ascii
from .base import TestImportBase, FixtureDir


class TestUnicode(TestImportBase):
    """Test case for unicode errors."""

    def test_exception_logging_unicode_error(self):
        """
        Test logging of duplicate row and UnicodeDecodeError.

        See issue https://github.com/yourlabs/django-cities-light/issues/61
        """
        fixture_dir = FixtureDir('unicode')
        self.import_data(
            fixture_dir,
            'kemerovo_country',
            'kemerovo_region',
            'kemerovo_subregion',
            'kemerovo_city',
            'kemerovo_translations'
        )

    def test_unidecode_warning(self):
        """
        Unidecode should get unicode object and not byte string.

        unidecode/__init__.py:46: RuntimeWarning: Argument <type 'str'> is not an unicode object.
        Passing an encoded string will likely have unexpected results.

        This means to_ascii should return unicode string too.
        """
        # Reset warning registry to trigger the test if the warning was already issued
        # See http://bugs.python.org/issue21724
        registry = getattr(unidecode, '__warningregistry__', None)
        if registry:
            registry.clear()

        with warnings.catch_warnings(record=True) as warns:
            warnings.simplefilter('always')

            self.import_data(
                FixtureDir('unicode'),
                'kemerovo_country',
                'kemerovo_region',
                'kemerovo_subregion',
                'kemerovo_city',
                'kemerovo_translations'
            )

            for w in warns[:]:
                warn = force_str(w.message)
                self.assertTrue("not an unicode object" not in warn, warn)

    def test_to_ascii(self):
        """Test to_ascii behavior."""
        self.assertEqual(to_ascii('République Françaisen'), 'Republique Francaisen')
        self.assertEqual(to_ascii('Кемерово'), 'Kemerovo')
        self.assertTrue(isinstance(to_ascii('Кемерово'), str))



# Generated by Django 1.11.8 on 2018-01-01 19:46
import autoslug.fields
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('cities_light', '0008_city_timezone'),
    ]

    operations = [
        migrations.CreateModel(
            name='SubRegion',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, max_length=200)),
                ('name_ascii', models.CharField(blank=True, db_index=True, max_length=200)),
                ('slug', autoslug.fields.AutoSlugField(editable=False, populate_from='name_ascii')),
                ('geoname_id', models.IntegerField(blank=True, null=True, unique=True)),
                ('alternate_names', models.TextField(blank=True, default='', null=True)),
                ('display_name', models.CharField(max_length=200)),
                ('geoname_code', models.CharField(blank=True, db_index=True, max_length=50, null=True)),
                ('country', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='cities_light.Country')),
                ('region', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='cities_light.Region')),
            ],
            options={
                'verbose_name': 'SubRegion',
                'verbose_name_plural': 'SubRegions',
                'ordering': ['name'],
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='city',
            name='subregion',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='cities_light.SubRegion'),
        ),
    ]



"""
Django migrations for cities_light app
"""

# Ensure the user is not using Django 1.7
try:
    from django.db import migrations  # noqa
except ImportError:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured('Django < 1.7 not supported')



"""
Settings for this application. The most important is TRANSLATION_LANGUAGES
because it's probably project specific.

.. py:data:: TRANSLATION_LANGUAGES

    List of language codes. It is used to generate the alternate_names property
    of cities_light models. You want to keep it as small as possible.
    By default, it includes the most popular languages according to wikipedia,
    which use a rather ascii-compatible alphabet. It also contains 'abbr' which
    stands for 'abbreviation', you might want to include this one as well.

    See:

     - http://download.geonames.org/export/dump/iso-languagecodes.txt

    Example::

        CITIES_LIGHT_TRANSLATION_LANGUAGES = ['es', 'en', 'fr', 'abbr']

.. py:data:: INCLUDE_COUNTRIES

    List of country codes to include. It's None by default which lets all
    countries in the database. But if you only wanted French and Belgium
    countries/regions/cities, you could set it as such::

        CITIES_LIGHT_INCLUDE_COUNTRIES = ['FR', 'BE']

.. py:data:: INCLUDE_CITY_TYPES

    List of city feature codes to include. They are described at
    http://www.geonames.org/export/codes.html, section "P city, village".

        CITIES_LIGHT_INCLUDE_CITY_TYPES = [
            'PPL', 'PPLA', 'PPLA2', 'PPLA3', 'PPLA4', 'PPLC',
            'PPLF', 'PPLG', 'PPLL', 'PPLR', 'PPLS', 'STLMT',
        ]

.. py:data:: COUNTRY_SOURCES

    A list of urls to download country info from. Default is countryInfo.txt
    from geonames download server. Overridable in
    ``settings.CITIES_LIGHT_COUNTRY_SOURCES``.

.. py:data:: REGION_SOURCES

    A list of urls to download region info from. Default is
    admin1CodesASCII.txt from geonames download server. Overridable in
    ``settings.CITIES_LIGHT_REGION_SOURCES``.

.. py:data:: SUBREGION_SOURCES

    A list of urls to download region info from. Default is
    admin2Codes.txt from geonames download server. Overridable in
    ``settings.CITIES_LIGHT_SUBREGION_SOURCES``.

.. py:data:: CITY_SOURCES

    A list of urls to download city info from. Default is cities15000.zip from
    geonames download server. Overridable in
    ``settings.CITIES_LIGHT_CITY_SOURCES``.

.. py:data:: TRANSLATION_SOURCES

    A list of urls to download alternate names info from. Default is
    alternateNames.zip from geonames download server. Overridable in
    ``settings.CITIES_LIGHT_TRANSLATION_SOURCES``.

.. py:data:: SOURCES

    A list with all sources, auto-generated.

.. py:data:: FIXTURES_BASE_URL

   Base URL to download country/region/city fixtures from. Should end
   with a slash. Default is ``file://DATA_DIR/fixtures/``. Overridable in
   ``settings.CITIES_LIGHT_FIXTURES_BASE_URL``.

.. py:data:: DATA_DIR

    Absolute path to download and extract data into. Default is
    cities_light/data. Overridable in ``settings.CITIES_LIGHT_DATA_DIR``

.. py:data:: INDEX_SEARCH_NAMES

    If your database engine for cities_light supports indexing TextFields (ie.
    it is **not** MySQL), then this should be set to True. You might have to
    override this setting with ``settings.CITIES_LIGHT_INDEX_SEARCH_NAMES`` if
    using several databases for your project.

.. py:data:: CITIES_LIGHT_APP_NAME

    Modify it only if you want to define your custom cities models, that
    are inherited from abstract models of this package.
    It must be equal to app name, where custom models are defined.
    For example, if they are in geo/models.py, then set
    ``settings.CITIES_LIGHT_APP_NAME = 'geo'``.
    Note: you can't define one custom model, you have to define all of
    cities_light models, even if you want to modify only one.
"""
import os.path

from django.conf import settings

__all__ = [
    'FIXTURES_BASE_URL', 'COUNTRY_SOURCES', 'REGION_SOURCES',
    'SUBREGION_SOURCES', 'CITY_SOURCES', 'TRANSLATION_LANGUAGES',
    'TRANSLATION_SOURCES', 'SOURCES', 'DATA_DIR', 'INDEX_SEARCH_NAMES',
    'INCLUDE_COUNTRIES', 'INCLUDE_CITY_TYPES', 'DEFAULT_APP_NAME',
    'CITIES_LIGHT_APP_NAME', 'ICountry', 'IRegion', 'ISubRegion', 'ICity',
    'IAlternate']

COUNTRY_SOURCES = getattr(settings, 'CITIES_LIGHT_COUNTRY_SOURCES',
    ['http://download.geonames.org/export/dump/countryInfo.txt'])
REGION_SOURCES = getattr(settings, 'CITIES_LIGHT_REGION_SOURCES',
    ['http://download.geonames.org/export/dump/admin1CodesASCII.txt'])
SUBREGION_SOURCES = getattr(settings, 'CITIES_LIGHT_SUBREGION_SOURCES',
    ['http://download.geonames.org/export/dump/admin2Codes.txt'])
CITY_SOURCES = getattr(settings, 'CITIES_LIGHT_CITY_SOURCES',
    ['http://download.geonames.org/export/dump/cities15000.zip'])
TRANSLATION_SOURCES = getattr(settings, 'CITIES_LIGHT_TRANSLATION_SOURCES',
    ['http://download.geonames.org/export/dump/alternateNames.zip'])
TRANSLATION_LANGUAGES = getattr(settings, 'CITIES_LIGHT_TRANSLATION_LANGUAGES',
    ['es', 'en', 'pt', 'de', 'pl', 'abbr'])

SOURCES = list(COUNTRY_SOURCES) + list(REGION_SOURCES) + \
    list(SUBREGION_SOURCES) + list(CITY_SOURCES)
SOURCES += TRANSLATION_SOURCES

DATA_DIR = getattr(settings, 'CITIES_LIGHT_DATA_DIR',
    os.path.normpath(os.path.join(
        os.path.dirname(os.path.realpath(__file__)), 'data')))

INCLUDE_COUNTRIES = getattr(settings, 'CITIES_LIGHT_INCLUDE_COUNTRIES', None)

# Feature codes are described in the "P city, village" section at
# http://www.geonames.org/export/codes.html
INCLUDE_CITY_TYPES = getattr(
    settings,
    'CITIES_LIGHT_INCLUDE_CITY_TYPES',
    ['PPL', 'PPLA', 'PPLA2', 'PPLA3', 'PPLA4', 'PPLC',
     'PPLF', 'PPLG', 'PPLL', 'PPLR', 'PPLS', 'STLMT']
)

# MySQL doesn't support indexing TextFields
INDEX_SEARCH_NAMES = getattr(settings, 'CITIES_LIGHT_INDEX_SEARCH_NAMES', None)
if INDEX_SEARCH_NAMES is None:
    INDEX_SEARCH_NAMES = True
    for database in list(settings.DATABASES.values()):
        if "ENGINE" in database and 'mysql' in database.get('ENGINE').lower():
            INDEX_SEARCH_NAMES = False

DEFAULT_APP_NAME = 'cities_light'
CITIES_LIGHT_APP_NAME = getattr(settings, 'CITIES_LIGHT_APP_NAME',
                                DEFAULT_APP_NAME)

FIXTURES_BASE_URL = getattr(
    settings,
    'CITIES_LIGHT_FIXTURES_BASE_URL',
    'file://{0}'.format(os.path.join(DATA_DIR, 'fixtures/'))
)


class ICountry:
    """
    Country field indexes in geonames.
    """
    code2 = 0
    code3 = 1
    codeNum = 2
    fips = 3
    name = 4
    capital = 5
    area = 6
    population = 7
    continent = 8
    tld = 9
    currencyCode = 10
    currencyName = 11
    phone = 12
    postalCodeFormat = 13
    postalCodeRegex = 14
    languages = 15
    geonameid = 16
    neighbours = 17
    equivalentFips = 18


class IRegion:
    """
    Region field indexes in geonames.
    """
    code = 0
    name = 1
    asciiName = 2
    geonameid = 3


class ISubRegion:
    """
    Subregion field indexes in geonames.
    """
    code = 0
    name = 1
    asciiName = 2
    geonameid = 3


class ICity:
    """
    City field indexes in geonames.
    Description of fields: http://download.geonames.org/export/dump/readme.txt
    """
    geonameid = 0
    name = 1
    asciiName = 2
    alternateNames = 3
    latitude = 4
    longitude = 5
    featureClass = 6
    featureCode = 7
    countryCode = 8
    cc2 = 9
    admin1Code = 10
    admin2Code = 11
    admin3Code = 12
    admin4Code = 13
    population = 14
    elevation = 15
    gtopo30 = 16
    timezone = 17
    modificationDate = 18


class IAlternate:
    """
    Alternate names field indexes in geonames.
    Description of fields: http://download.geonames.org/export/dump/readme.txt
    """
    nameid = 0
    geonameid = 1
    language = 2
    name = 3
    isPreferred = 4
    isShort = 5
    isColloquial = 6
    isHistoric = 7



from .settings import CITIES_LIGHT_APP_NAME
from django.apps import apps

get_model = apps.get_model


def get_cities_model(model_name: str, *args, **kwargs):
    """
    Returns cities model, either default either customised, depending on
    ``settings.CITIES_LIGHT_APP_NAME``.
    """
    return get_model(CITIES_LIGHT_APP_NAME, model_name, *args, **kwargs)


def get_cities_models(model_names: tuple = ('Country', 'Region', 'SubRegion',
                                            'City')):
    return [get_cities_model(model_name) for model_name in model_names]



import re
import autoslug
import pytz

from django.db import models
from django.db.models import lookups
from django.utils.encoding import force_str
from django.conf import settings
from django.utils.translation import gettext_lazy as _

from unidecode import unidecode

from .validators import timezone_validator
from .settings import INDEX_SEARCH_NAMES, CITIES_LIGHT_APP_NAME

__all__ = ['AbstractCountry', 'AbstractRegion',
           'AbstractSubRegion', 'AbstractCity',
           'CONTINENT_CHOICES']

CONTINENT_CHOICES = (
    ('OC', _('Oceania')),
    ('EU', _('Europe')),
    ('AF', _('Africa')),
    ('NA', _('North America')),
    ('AN', _('Antarctica')),
    ('SA', _('South America')),
    ('AS', _('Asia')),
)

ALPHA_REGEXP = re.compile(r'[\W_]+', re.UNICODE)


def to_ascii(value):
    """
    Convert a unicode value to ASCII-only unicode string.

    For example, 'République Françaisen' would become 'Republique Francaisen'
    """
    return force_str(unidecode(value))


def to_search(value):
    """
    Convert a string value into a string that is usable against
    City.search_names.

    For example, 'Paris Texas' would become 'paristexas'.
    """

    return ALPHA_REGEXP.sub('', to_ascii(value)).lower()


class ToSearchIContainsLookup(lookups.IContains):
    """IContains lookup for ToSearchTextField."""

    def get_prep_lookup(self):
        """Return the value passed through to_search()."""
        value = super().get_prep_lookup()
        return to_search(value)


class ToSearchTextField(models.TextField):
    """
    Trivial TextField subclass that passes values through to_search
    automatically.
    """


ToSearchTextField.register_lookup(ToSearchIContainsLookup)


class BaseManager(models.Manager):
    def get_by_natural_key(self, geoname_id):
        return self.get(geoname_id=geoname_id)


class Base(models.Model):
    """
    Base model with boilerplate for all models.
    """

    name = models.CharField(max_length=200, db_index=True)
    name_ascii = models.CharField(max_length=200, blank=True, db_index=True)
    slug = autoslug.AutoSlugField(populate_from='name_ascii')
    geoname_id = models.IntegerField(null=True, blank=True, unique=True)
    alternate_names = models.TextField(null=True, blank=True, default='')

    objects = BaseManager()

    class Meta:
        abstract = True
        ordering = ['name']

    def natural_key(self):
        return (self.geoname_id, )

    def __str__(self):
        display_name = getattr(self, 'display_name', None)
        if display_name:
            return display_name
        return self.name


class AbstractCountry(Base):
    """
    Base Country model.
    """

    code2 = models.CharField(max_length=2, null=True, blank=True, unique=True)
    code3 = models.CharField(max_length=3, null=True, blank=True, unique=True)
    continent = models.CharField(max_length=2, db_index=True,
                                 choices=CONTINENT_CHOICES)
    tld = models.CharField(max_length=5, blank=True, db_index=True)
    phone = models.CharField(max_length=20, null=True, blank=True)

    class Meta(Base.Meta):
        verbose_name_plural = _('countries')
        abstract = True


class AbstractRegion(Base):
    """
    Base Region/State model.
    """

    display_name = models.CharField(max_length=200)
    geoname_code = models.CharField(max_length=50, null=True, blank=True,
                                    db_index=True)

    country = models.ForeignKey(CITIES_LIGHT_APP_NAME + '.Country',
                                on_delete=models.CASCADE)

    class Meta(Base.Meta):
        unique_together = (('country', 'name'), ('country', 'slug'))
        verbose_name = _('region/state')
        verbose_name_plural = _('regions/states')
        abstract = True

    def get_display_name(self):
        return '%s, %s' % (self.name, self.country.name)


class AbstractSubRegion(Base):
    """
    Base SubRegion model.
    """

    display_name = models.CharField(max_length=200)
    geoname_code = models.CharField(max_length=50, null=True, blank=True,
                                    db_index=True)

    country = models.ForeignKey(CITIES_LIGHT_APP_NAME + '.Country',
                                on_delete=models.CASCADE)
    region = models.ForeignKey(CITIES_LIGHT_APP_NAME + '.Region',
                               null=True, blank=True,
                               on_delete=models.CASCADE)

    class Meta(Base.Meta):
        verbose_name = _('SubRegion')
        verbose_name_plural = _('SubRegions')
        abstract = True

    def get_display_name(self):
        return '%s, %s' % (self.name, self.country.name)


class AbstractCity(Base):
    """
    Base City model.
    """

    display_name = models.CharField(max_length=200)

    search_names = ToSearchTextField(
        max_length=4000,
        db_index=INDEX_SEARCH_NAMES,
        blank=True,
        default='')

    latitude = models.DecimalField(
        max_digits=8,
        decimal_places=5,
        null=True,
        blank=True)

    longitude = models.DecimalField(
        max_digits=8,
        decimal_places=5,
        null=True,
        blank=True)

    subregion = models.ForeignKey(CITIES_LIGHT_APP_NAME + '.SubRegion',
                                  blank=True, null=True,
                                  on_delete=models.CASCADE)
    region = models.ForeignKey(CITIES_LIGHT_APP_NAME + '.Region', blank=True,
                               null=True, on_delete=models.CASCADE)
    country = models.ForeignKey(CITIES_LIGHT_APP_NAME + '.Country',
                                on_delete=models.CASCADE)
    population = models.BigIntegerField(null=True, blank=True, db_index=True)
    feature_code = models.CharField(max_length=10, null=True, blank=True,
                                    db_index=True)
    timezone = models.CharField(max_length=40, blank=True, null=True,
                                db_index=True, validators=[timezone_validator])

    class Meta(Base.Meta):
        unique_together = (('region', 'subregion', 'name'),
                           ('region', 'subregion', 'slug'))
        verbose_name_plural = _('cities')
        abstract = True

    def get_display_name(self):
        if self.region_id:
            return '%s, %s, %s' % (self.name, self.region.name,
                                   self.country.name)
        else:
            return '%s, %s' % (self.name, self.country.name)

    def get_timezone_info(self):
        """Return timezone info for self.timezone.

        If self.timezone has wrong value, it returns timezone info
        for value specified in settings.TIME_ZONE.
        """
        try:
            return pytz.timezone(self.timezone)
        except (pytz.UnknownTimeZoneError, AttributeError):
            return pytz.timezone(settings.TIME_ZONE)



import glob
import os

from dbdiff.fixture import Fixture
from .base import TestImportBase, FixtureDir
from ..settings import DATA_DIR


class TestImport(TestImportBase):
    """Load test."""

    def test_single_city(self):
        """Load single city."""
        fixture_dir = FixtureDir('import')
        self.import_data(
            fixture_dir,
            'angouleme_country',
            'angouleme_region',
            'angouleme_subregion',
            'angouleme_city',
            'angouleme_translations'
        )
        Fixture(fixture_dir.get_file_path('angouleme.json')).assertNoDiff()

    def test_single_city_zip(self):
        """Load single city."""
        filelist = glob.glob(os.path.join(DATA_DIR, "angouleme_*.txt"))
        for f in filelist:
            os.remove(f)

        fixture_dir = FixtureDir('import_zip')
        self.import_data(
            fixture_dir,
            'angouleme_country',
            'angouleme_region',
            'angouleme_subregion',
            'angouleme_city',
            'angouleme_translations',
            file_type="zip"
        )
        Fixture(FixtureDir('import').get_file_path('angouleme.json')).assertNoDiff()

    def test_city_wrong_timezone(self):
        """Load single city with wrong timezone."""
        fixture_dir = FixtureDir('import')
        self.import_data(
            fixture_dir,
            'angouleme_country',
            'angouleme_region',
            'angouleme_subregion',
            'angouleme_city_wtz',
            'angouleme_translations'
        )
        Fixture(fixture_dir.get_file_path('angouleme_wtz.json')).assertNoDiff()

        from ..loading import get_cities_model
        city_model = get_cities_model('City')
        cities = city_model.objects.all()
        for city in cities:
            print(city.get_timezone_info().zone)

